#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "pool4xi.py">
#<abstract4xi abstract = "provides class Pool4Xi"/>
#<refer4xi reference = "pool.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Pool4Xi' ]  
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import collections                 
    import itertools
    import math
    import queue
    import time         
except:
    pass
try:                 # rare ones
    pass          
except:
    pass 
try:                 # imported-as ones 
    pass          
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.pool import Pool
except:
    print("pool4xi.py *** failed in from multiprocessing.pool import Pool ***")
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from applyresult4xi import ApplyResult4Xi

from constant_pool4xi import ConstantPool4Xi

from cpu_corecount4xi import cpu_corecount4xi

from finalize4xi import Finalize4Xi

from imapiterator4xi import IMapIterator4Xi 
from imapunorderediterator4xi import IMapUnorderedIterator4Xi

from mapresult4xi import MapResult4Xi
from mapstar4xi import mapstar4xi

from print4xi import print4xi
from process4xi import Process4Xi

from queue4xi import Queue4Xi

from reason_object4xi import reason_object4xi

from sense_object4xi import sense_object4xi
from simplequeue4xi import SimpleQueue4Xi

from terminate4xi import terminate4xi
from time4xi import time4xi

from worker4xi import worker4xi
#"
#/>


# =================================================================================================
#<class4xi class = "Pool4Xi" base = "object" child = "None" synopsis = "representing a process pool" usage = 
#"
#   managers.py
#       SyncManager.register('Pool', Pool, PoolProxy)
#   syncmanager4xi.py:
#       SyncManager4Xi.register('Pool4Xi', Pool4Xi, PoolProxy4Xi)
#   threadpool4xi.py:
#       class ThreadPool4Xi(Pool4Xi):
#   test_ctypes_mp_xi4.py:
#       pool2xi = Pool4Xi(processes=2)
#   test_pool1_mp_xi4.py:
#       pool8xi = Pool4Xi(processes=8)
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
#
class Pool4Xi(object):
    '''
    represents a process pool
    '''
    
    
    # ---------------------------------------------------------------------------------------------
    # Public API of this class 
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('apply', 'map', 'imap', 'imap_unordered',
                 'apply_async', 'map_async',
                 'close', 'terminate', 'join','starmap', 'starmap_async',                
                 'sense', 'reason', 'ace')        
    #"
    #/>    
        
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    Process = Process4Xi 
    _cp2xi = ConstantPool4Xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Pool4Xi" usage = "initializes Pool4Xi">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------                              
    def __init__(self, processes=None, initializer=None, initargs=(), maxtasksperchild=None):
        '''
        initializes Pool4Xi    
        '''
            
        # -----------------------------------------------------------------------------------------
        # (Step 1/23) - INITIALIZE           
        # ----------------------------------------------------------------------------------------- 
             
        class2print = "Pool4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/23a) started" + "; init-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #23:42:47 01/15/13 Pacific Standard Time
                        
        message2print = "{(Step 1/23b) borning" + "; pool2xi-of-Pool4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #23:42:47 01/15/13 Pacific Standard Time
                     
        message2print = "{(Step 1/23c) inheriting" + "; object-inheriting-by-pool2xi-of-Pool4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #<class 'object'>
                        
        message2print = "{(Step 1/23d) sharing" + "; Process-sharing-by-pool2xi-of-Pool4Xi: " + str(Pool4Xi.Process)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        #<class 'process4xi.Process4Xi'>
        
        message2print = "{(Step 1/23e) sharing" + "; constantpool2xi-sharing-by-pool2xi-of-Pool4Xi: " + str(Pool4Xi._cp2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        #<constant_pool4xi.ConstantPool4Xi object at 0x000000000370E4A8>
                                
        message2print = "{(Step 1/23f) passed" + "; processes-passed-to-init-of-Pool4Xi: " + str(processes) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #2;4;8
                     
        message2print = "{(Step 1/23g) passed" + "; initializer-passed-to-init-of-Pool4Xi: " + str(initializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #None
                     
        message2print = "{(Step 1/23h) passed" + "; initargs-passed-to-init-of-Pool4Xi: " + str(initargs) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #()
        
        message2print = "{(Step 1/23i) passed" + "; maxtasksperchild-passed-to-init-of-Pool4Xi: " + str(maxtasksperchild) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #None
            
        # -----------------------------------------------------------------------------------------
        # (Step 2/23) - SETUP QUEUES          
        # ----------------------------------------------------------------------------------------- 
        try:
            self._setup_queues()                                        # in and out locked Pipes                                     
            
            message2print = "{(Step 2/23a) performed" + "; setup-queues-of-pool2xi-performed-by-init-of-Pool4Xi: " + str(self._setup_queues) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #<bound method Pool4Xi._setup_queues of <pool4xi.Pool4Xi object at 0x0000000003A5D9B0>>
                         
        except:
            message2print = "{(Step 2/23b) hit" + "; setup-queues-of-pool2xi-hit-by-init-of-Pool4Xi: " + " *** " + str(self._setup_queues)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                      
        # -----------------------------------------------------------------------------------------    
        # (Step 3/23) - TASK QUEUE
        # -----------------------------------------------------------------------------------------     
        self._taskqueue = None
        try:      
            self._taskqueue = queue.Queue()                             # standard queues for tasks
            
            message2print = "{(Step 3/23) set" + "; taskqueue-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._taskqueue)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #<queue.Queue object at 0x0000000003A5DB70>
        except:
            message2print = "{(Step 3/23z) hit" + "; taskqueue-of-pool2xi-hit-by-init-of-Pool4Xi: " + " *** " + str(self._taskqueue)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
    
        # -----------------------------------------------------------------------------------------    
        # (Step 4/23) - CACHE
        # -----------------------------------------------------------------------------------------     
        self._cache = {}
        
        message2print = "{(Step 4/23) set" + "; cache-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._cache)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #{}
           
    
        # -----------------------------------------------------------------------------------------    
        # (Step 5/23) - STATE
        # -----------------------------------------------------------------------------------------     
        self._state = Pool4Xi._cp2xi.get_run()                           # RUN_POOL4XI dragonized 
        
        message2print = "{(Step 5a/23) set" + "; state-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._state)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #0
    
        # -----------------------------------------------------------------------------------------    
        # (Step 5/23) - MAX TASKS PER CHILD
        # -----------------------------------------------------------------------------------------         
        self._maxtasksperchild = maxtasksperchild
        message2print = "{(Step 5b/23) set" + "; maxtasksperchild-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._maxtasksperchild)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #None 
    
        # -----------------------------------------------------------------------------------------    
        # (Step 5/23) - INITIALIZER
        # -----------------------------------------------------------------------------------------                
        self._initializer = initializer
        message2print = "{(Step 5c/23) set" + "; initializer-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._initializer)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #None
         
        # -----------------------------------------------------------------------------------------    
        # (Step 5/23) - INITIALIZING ARGUMENTS
        # -----------------------------------------------------------------------------------------                
        self._initargs = initargs
        message2print = "{(Step 5d/23) set" + "; initargs-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._initargs)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #() 
        
           
        # -----------------------------------------------------------------------------------------    
        # (Step 6/23) - PROCESSES
        # -----------------------------------------------------------------------------------------     
        if processes is None:
            processes = 1
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            try:
                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                processes = cpu_corecount4xi()                           # with quad-core cpu four processes if input parameter=None
                
                message2print = "{(Step 6/23a) set" + "; processes-set-by-init-of-Pool4Xi to: " + str(processes)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #       
            except NotImplementedError:

                # ---------------------------------------------------------------------------------    
                # 
                # ---------------------------------------------------------------------------------     
                processes = 1
                
                message2print = "{(Step 6/23b) set" + "; processes-set-by-init-of-Pool4Xi to: " + str(processes)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                 
        else:
            message2print = "{(Step 6/23c) using" + "; processes-using-by-init-of-Pool4Xi to: " + str(processes)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 2
            
        # -----------------------------------------------------------------------------------------    
        # (Step 6/23) - PROCESSES TO OBJECT
        # -----------------------------------------------------------------------------------------         
        self._processes = processes
        message2print = "{(Step 6/23d) set" + "; processes-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._processes)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #2 
            
        # -----------------------------------------------------------------------------------------
        # (Step 7/23) - POOL TO OBJECT        
        # -----------------------------------------------------------------------------------------
        self._pool = []
        
        message2print = "{(Step 7/23) set" + "; pool-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._pool)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #[]
                        
    
        # -----------------------------------------------------------------------------------------    
        # (Step 8/23) - REPOPULATE OBJECT POOL 
        # -----------------------------------------------------------------------------------------     
        ack2repopulate = False
        try:
            self._repopulate_pool()
            
            message2print = "{(Step 8/23) performed" + "; repopulate-of-pool2xi-performed-by-init-of-Pool4Xi to: " + str(self._repopulate_pool())  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            ack2repopulate = True
        except:                         
            message2print = "{(Step 8/23z) hit" + "; ack2repopulate-hit-by-init-of-Pool4Xi: " + " *** " + str(ack2repopulate)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                    
    
        # -----------------------------------------------------------------------------------------    
        # (Step 9/23) - WORKER-HANDLER TO OBJECT 
        # -----------------------------------------------------------------------------------------        
        self._worker_handler = None
        ack4task = False
        try:
            self._worker_handler = threading.Thread(
                target=Pool4Xi._handle_workers,
                args=(self, )
                )
            message2print = "{(Step 9/23a) set" + "; worker-handler-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._worker_handler)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #<Thread(Thread-1, initial)> 
            
            ack4task = True
        except:                         
            message2print = "{(Step 9/23z) hit" + "; worker-handler-of-pool2xi-hit-by-init-of-Pool4Xi: " + " *** " + str(self._worker_handler)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
        
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 10/23) - WORKER HANDLER DAEMON TO OBJECT        
        # -----------------------------------------------------------------------------------------
        self._worker_handler.daemon = False 
        if ack4task is True:                       
                   
            self._worker_handler.daemon = True
            
            message2print = "{(Step 10/23a) set" + "; worker-handler-daemon-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._worker_handler.daemon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # True
                
        else:
            message2print = "{(Step 10/23z) warned" + "; worker-handler-daemon-of-pool2xi-warned-by-init-of-Pool4Xi: " + " ** WORKER-HANDLER-DAEMON NOT SET - NO TASK AVAILABLE   ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
                                
                       
    
        # -----------------------------------------------------------------------------------------    
        # (Step 11/23) - WORKER HANDLER STATE TO OBJECT   
        # -----------------------------------------------------------------------------------------     
        if ack4task is True:                       
             
            self._worker_handler._state = Pool4Xi._cp2xi.get_run()                 #RUN_POOL4XI dragonized
            
            message2print = "{(Step 11/23a) set" + "; worker-handler-state-of-pool2xi-set-by-init-of-Pool4Xi to: " + str(self._worker_handler._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #0
                
        else:
            message2print = "{(Step 11/23b) warned" + "; task-handler-state-of-pool2xi-warned-by-init-of-Pool4Xi: " + " ** TASK-HANDLER-STATE NOT SET - NO TASK AVAILABLE  ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            self._task_handler._state = Pool4Xi._cp2xi.get_terminate()
                             
    
        # -----------------------------------------------------------------------------------------    
        # (Step 12/23) - START OBJECT WORKER HANDLER
        # -----------------------------------------------------------------------------------------     
        if ack4task is True:                       
                     
            self._worker_handler.start()
            
            message2print = "{(Step 12/23a) performed" + "; worker-handler-start-of-pool2xi-performed-by-init-of-Pool4Xi: " + str(self._worker_handler.start) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #<bound method Thread.start of <Thread(Thread-1, started daemon 3660)>> 
                         
        else:
            message2print = "{(Step 12/23b) warned" + "; task-start-of-pool2xi-warned-by-init-of-Pool4Xi: " + " ** TASK NOT STARTED - NO TASK AVAILABLE ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
                       
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 13/23) - THREAD POOL HANDLE TASK FUNCTION FOR TASK HANDLER OF OBJECT        
        # -----------------------------------------------------------------------------------------
        if ack4task is True:                       
                         
            message2print = "{(Step 13/23a) passing" + "; handle-tasks-passing-to-threading-Thread-by-init-of-Pool4Xi: " + str(Pool4Xi._handle_tasks) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #<function Pool4Xi._handle_tasks at 0x0000000003B32048> 
                     
            message2print = "{(Step 13/23b) passing" + "; taskqueue-of-pool2xi-passing-to-threading-Thread-by-init-of-Pool4Xi: " + str(self._taskqueue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                
            #<queue.Queue object at 0x0000000003DD2C50> 
            
            message2print = "{(Step 13/23c) passing" + "; quick-put-of-pool2xi-passing-to-threading-Thread-by-init-of-Pool4Xi: " + str(self._quick_put) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                
            #<bound method PipeConnection4Xi.send of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003DD2A90>> 
            
            message2print = "{(Step 13/23d) passing" + "; outqueue-of-pool2xi-passing-to-threading-Thread-by-init-of-Pool4Xi: " + str(self._outqueue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                
            #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003DD2B70> 

            message2print = "{(Step 13/23e) passing" + "; pool-of-pool2xi-passing-to-threading-Thread-by-init-of-Pool4Xi: " + str(self._pool) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                
            #[<Process4Xi(PoolWorker4Xi-4, stopped daemon)>, <Process4Xi(PoolWorker4Xi-5, stopped daemon)>] 
            
            self._task_handler = None
            try:
                          
                self._task_handler = threading.Thread(
                    target=Pool4Xi._handle_tasks,
                    args=(self._taskqueue, self._quick_put, self._outqueue, self._pool)
                    )
            
                message2print = "{(Step 13/23f) got" + "; task-handler-of-pool2xi-got-by-init-of-Pool4Xi: " + str(self._task_handler)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #<Thread(Thread-2, initial)>    
                    
            except:        
                message2print = "{(Step 13/23z1) failed" + "; task-handler-of-pool2xi-failed-by-init-of-Pool4Xi: " + str(self._task_handler) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
 
        else:
                         
            self._task_handler = None
            message2print = "{(Step 13/23z2) warned" + "; task-handler-of-pool2xi-warned-by-init-of-Pool4Xi: " + str(self._task_handler) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
               
        # -----------------------------------------------------------------------------------------    
        # (Step 14/23) - TASK_HANDLER.DAEMON
        # -----------------------------------------------------------------------------------------     
        if ack4task is True:                       
                                 
            self._task_handler.daemon = True
            
            message2print = "{(Step 14/23a) set" + "; task-handler-daemon-of-pool2xi-set-by-init-of-Pool4Xi: " + str(self._task_handler.daemon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # True
        else:
            self._task_handler.daemon = False    
            message2print = "{(Step 14/23z) warned" + "; task-handler-daemon-of-pool2xi-warned-by-init-of-Pool4Xi: "  + str(self._task_handler.daemon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
                        
    
        # -----------------------------------------------------------------------------------------    
        # (Step 15/23) - SET TASK HANDLER STATE TO RUN
        # -----------------------------------------------------------------------------------------     
        self._task_handler._state = Pool4Xi._cp2xi.get_run()
        if ack4task is True:                       
                                 
            self._task_handler._state =  Pool4Xi._cp2xi.get_run()               #RUN_POOL4XI dragonized
            
            message2print = "{(Step 15/23a) set" + "; task-handler-state-of-pool2xi-set-by-init-of-Pool4Xi: " + str(self._task_handler._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #0;
        else:
            self._task_handler._state = Pool4Xi._cp2xi.get_terminate()
            
            message2print = "{(Step 15/23z) set" + "; task-handler-state-of-pool2xi-set-by-init-of-Pool4Xi: " + str(self._task_handler._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        # -----------------------------------------------------------------------------------------    
        # (Step 16/23) - START TASK HANDLER
        # -----------------------------------------------------------------------------------------     
        if ack4task is True:                       
                              
            self._task_handler.start()
            
            message2print = "{(Step 16/23a) performed" + "; task-handler-start-of-pool2xi-performed-by-init-of-Pool4Xi: " + str(self._task_handler.start) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #<bound method Thread.start of <Thread(Thread-2, started daemon 3048)>>
        else:
            message2print = "{(Step 16/23b) skipped" + "; task-handler-start-of-pool2xi-skipped-by-init-of-Pool4Xi: " + str(self._task_handler.start) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
        # -----------------------------------------------------------------------------------------    
        # (Step 17/23) - GET RESULT WITH THREAD
        # -----------------------------------------------------------------------------------------     
        self._result_handler = None
        if ack4task is True:                       
            message2print = "{(Step 17/23a) passing" + "; outqueue-of-pool2xi-passing-to-threading-Thread-by-init-of-Pool4Xi: " + str(self._outqueue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003DD2B70> 
            
            message2print = "{(Step 17/23b) passing" + "; quick-get-of-pool2xi-passing-to-threading-Thread-by-init-of-Pool4Xi: " + str(self._quick_get) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #<bound method PipeConnection4Xi.recv of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003DD2C18>> 
            
            message2print = "{(Step 17/23c) passing" + "; cache-of-pool2xi-passing-to-threading-Thread-by-init-of-Pool4Xi: " + str(self._cache) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #{}   
                            
            self._result_handler = threading.Thread(
                target=Pool4Xi._handle_results,
                args=(self._outqueue, self._quick_get, self._cache)
            )
            
            message2print = "{(Step 17/23d) got" + "; result-handler-of-pool2xi-got-by-init-of-Pool4Xi: " + str(self._result_handler) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #<Thread(Thread-3, initial)>
        else:
            message2print = "{(Step 17/23z) failed" + "; result-handler-of-pool2xi-failed-by-init-of-Pool4Xi: " + str(self._result_handler) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
        # -----------------------------------------------------------------------------------------    
        # (Step 18/23) - DAEMON OF RESULT
        # -----------------------------------------------------------------------------------------     
        if self._result_handler is None:
            message2print = "{(Step 18/23z) skipped" + "; result-handler-of-pool2xi-skipped-by-init-of-Pool4Xi: " + str(self._result_handler)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #   
        else:
            self._result_handler.daemon = True
            
            message2print = "{(Step 18/23a) set" + "; result-handler-daemon-of-pool2xi-set-by-init-of-Pool4Xi: " + str(self._result_handler.daemon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #True
             
        # -----------------------------------------------------------------------------------------    
        # (Step 19/23) - STATE OF RESULT
        # -----------------------------------------------------------------------------------------     
        if self._result_handler is None:
            message2print = "{(Step 19/23z) skipped" + "; result-handler-of-pool2xi-skipped-by-init-of-Pool4Xi: " + str(self._result_handler)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #   
        else:
            self._result_handler._state = Pool4Xi._cp2xi.get_run()  
            
            message2print = "{(Step 19/23a) set" + "; daemon-of-pool2xi-set-by-init-of-Pool4Xi: " + str(self._result_handler.daemon)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #True
                                             
        # -----------------------------------------------------------------------------------------    
        # (Step 20/23) - START WITH RESULT HANDLER
        # -----------------------------------------------------------------------------------------     
        if self._result_handler is None:
            message2print = "{(Step 20/23z1) skipped" + "; result-handler-of-pool2xi-skipped-by-init-of-Pool4Xi: " + str(self._result_handler)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #   
        else:
            try:
                self._result_handler.start()
                
                message2print = "{(Step 20/23a) performed" + "; result-handler-start-performed-by-init-of-Pool4Xi: " + str(self._result_handler.start)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #<bound method Thread.start of <Thread(Thread-3, started daemon 2840)>> 
            except:
                message2print = "{(Step 20/23z2) failed" + "; result-handler-start-failed-by-init-of-Pool4Xi: " + str(self._result_handler.start)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                                        
      
        # -----------------------------------------------------------------------------------------
        # (Step 22/23) - FINALIZE 
        # -----------------------------------------------------------------------------------------

        message2print = "{(Step 22/23a) passing" + "; pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #00:30:01 01/16/13 Pacific Standard Time
        
        message2print = "{(Step 22/23b) passing" + "; terminate-pool-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._terminate_pool) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #<bound method type._terminate_pool of <class 'pool4xi.Pool4Xi'>>

        message2print = "{(Step 22/23c) passing" + "; taskqueue-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._taskqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #<queue.Queue object at 0x0000000003DD2C50>
        
        message2print = "{(Step 22/23d) passing" + "; inqueue-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._inqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003DD2AC8>
                
        message2print = "{(Step 22/23e) passing" + "; outqueue-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._outqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003DD2B70>
                
        message2print = "{(Step 22/23f) passing" + "; pool-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._pool) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #[<Process4Xi(PoolWorker4Xi-4, stopped daemon)>, <Process4Xi(PoolWorker4Xi-5, stopped daemon)>]
                
        message2print = "{(Step 22/23g) passing" + "; worker-handler-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._worker_handler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #<Thread(Thread-1, started daemon 3660)> 
        
        message2print = "{(Step 22/23h) passing" + "; task-handler-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._task_handler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #<Thread(Thread-2, started daemon 3048)>
                
        message2print = "{(Step 22/23i) passing" + "; result-handler-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._result_handler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #<Thread(Thread-3, started daemon 2840)>
                
        message2print = "{(Step 22/23j) passing" + "; cache-of-pool2xi-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(self._cache) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #{}
                          
        message2print = "{(Step 22/23k) passing" + "; exitpriority-passing-to-Finalize4Xi-by-init-of-Pool4Xi: " + str(Pool4Xi._cp2xi.get_exitpriority()) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #15
                  
        try:
            self._terminate = Finalize4Xi(
                self, self._terminate_pool,
                args=(self._taskqueue, self._inqueue, self._outqueue, self._pool,
                      self._worker_handler, self._task_handler,
                      self._result_handler, self._cache),
                exitpriority=15
            )
        
            message2print = "{(Step 22/23l) got" + "; terminate-of-pool2xi-got-by-init-of-Pool4Xi: " + str(self._terminate)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)
            #<finalize4xi: __repr__: Finalize object, callback=_terminate_pool, 
            #    args=(<queue.Queue object at 0x0000000003C0EC50>, <simplequeue4xi.SimpleQueue4Xi object at 0x0000000003C0E160>, <simplequeue4xi.SimpleQueue4Xi object at 0x0000000003C0EB70>, 
            #          [<Process4Xi(PoolWorker4Xi-4, stopped daemon)>], <Thread(Thread-1, started daemon 3304)>, <Thread(Thread-2, started daemon 1052)>, <Thread(Thread-3, started daemon 3684)>,
            #            {}), exitprority=15>; time: 01:01:53 01/16/13 Pacific Standard Time}

        
        except:
            message2print = "{(Step 22/23m) hit" + "; Finalize4Xi-hit-by-init-of-Pool4Xi: " + " *** " + str(Finalize4Xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
               
        # -----------------------------------------------------------------------------------------
        # (Step 23/23)            
        # -----------------------------------------------------------------------------------------
                     
        message2print = "{(Step 23/23a) inherited" + "; object-inherited-by-pool2xi-of-Pool4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #<class 'object'>
                    
        message2print = "{(Step 24/23b) activated" + "; pool2xi-of-Pool4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #00:30:02 01/16/13 Pacific Standard Time
        
                                
        message2print = "{(Step 1/23c) shared" + "; Process-shared-by-pool2xi-of-Pool4Xi: " + str(Pool4Xi.Process)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        #<class 'process4xi.Process4Xi'>
                                      

        message2print = "{(Step 24/23d) returning" + "; init-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #00:30:02 01/16/13 Pacific Standard Time
                                     
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "join-exited-workers-of-Pool4Xi" usage = "
    #   (self)
    #">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _join_exited_workers(self):
        '''
        Cleanup after any worker processes which have exited due to reaching
        their specified lifetime.  Returns True if any workers were cleaned2xi up.
        ''' 
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/4) 
        # -----------------------------------------------------------------------------------------     
        class2print = "Pool4Xi"
        method2print = "_join_exited_workers"
        
        message2print = "{ (Step 1/4) started" + "; join-exited-workers-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #00:30:01 01/16/13 Pacific Standard Time
        

        # -----------------------------------------------------------------------------------------    
        # (Step 2/4)  
        # -----------------------------------------------------------------------------------------                 
        cleaned2xi = False

        message2print = "{(Step 2/4) set" + "; cleaned2xi-set-by-join-exited-workers-of-Pool4Xi to: " + str(cleaned2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #False
                         

        # -----------------------------------------------------------------------------------------    
        # (Step 3/4) 
        # -----------------------------------------------------------------------------------------                 
        try:
            for i2xi in reversed(range(len(self._pool))):
                try:
                    worker2xi = self._pool[i2xi]
                    
                    message2print = "{(Step 3/4a) got" + "; worker2xi-got-by-join-exited-workers-of-Pool4Xi: " + str(worker2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #<Process4Xi(PoolWorker4Xi-5, stopped daemon)>
                    #<Process4Xi(PoolWorker4Xi-4, stopped daemon)>
                    
                    if worker2xi.exitcode is not None:
                        # worker2xi exited
                                                
                        worker2xi.join()
                        message2print = "{(Step 3/4b) performed" + "; join-performed-by-join-exited-workers-of-Pool4Xi: " + str(worker2xi.join)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #<bound method Process4Xi.join of <Process4Xi(PoolWorker4Xi-5, stopped daemon)>>
                        #<bound method Process4Xi.join of <Process4Xi(PoolWorker4Xi-4, stopped daemon)>>
                        
                        cleaned2xi = True
                        
                        message2print = "{(Step 3/4c) deleting" + "; index-deleting-by-join-exited-workers-of-Pool4Xi: " + str(i2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #1;0
                        
                        del self._pool[i2xi]
                    else:
                        message2print = "{(Step 3/4d) skipped" + "; worker2xi-skipped-by-join-exited-workers-of-Pool4Xi: " + str(worker2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #
                            
                except:
                    message2print = "{(Step 3/4z2) failed" + "; cleaned2xi-failed-by-join-exited-workers-of-Pool4Xi: " + str(cleaned2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #            
        except:
            message2print = "{(Step 3/4z1) failed" + "; cleaned2xi-failed-by-join-exited-workers-of-Pool4Xi: " + str(cleaned2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #       

        # -----------------------------------------------------------------------------------------    
        #(Step 4/4) 
        # -----------------------------------------------------------------------------------------                 
        
        message2print = "{(Step 4/4) returning" + "; cleaned2xi-returning-by-join-exited-workers-of-Pool4Xi: " + str(cleaned2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #True
        
        return cleaned2xi                
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "repopulate-pool-of-Pool4Xi" usage = "
    #   (self)
    #">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _repopulate_pool(self):
        '''
        Bring the number of pool processes up to the specified number,
        for use after reaping workers which have exited.
        ''' 
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3) 
        # -----------------------------------------------------------------------------------------     
        class2print = "Pool4Xi"
        method2print = "_repopulate_pool"
        
        message2print = "{ (Step 1/3) started" + "; repopulate-pool-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #23:42:47 01/15/13 Pacific Standard Time; 00:30:02 01/16/13 Pacific Standard Time
        

        # -----------------------------------------------------------------------------------------    
        # (Step 3/3) 
        # -----------------------------------------------------------------------------------------                 
        w2xi = None
        try:
            for i2xi in range(self._processes - len(self._pool)):
                w2xi = None
                try:
                    message2print = "{(Step 3/3a) passing" + "; worker4xi-of-pool2xi-passing-to-Process4Xi-by-repopulate-pool-of-Pool4Xi: " + str(worker4xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #<function worker4xi at 0x0000000003E0CD90>; <function worker4xi at 0x0000000003E0CD90>; <function worker4xi at 0x0000000003E0CD90>
                    
                    message2print = "{(Step 3/3b) passing" + "; inqueue-of-pool2xi-passing-to-Process4Xi-by-repopulate-pool-of-Pool4Xi: " + str(self._inqueue) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003A5D9E8>
                     
                    message2print = "{(Step 3/3c) passing" + "; outqueue-of-pool2xi-passing-to-Process4Xi-by-repopulate-pool-of-Pool4Xi: " + str(self._outqueue) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003A5DA90>
                     
                    message2print = "{(Step 3/3d) passing" + "; initializer-of-pool2xi-passing-to-Process4Xi-by-repopulate-pool-of-Pool4Xi: " + str(self._initializer) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #None
                     
                    message2print = "{(Step 3/3e) passing" + "; initargs-of-pool2xi-passing-to-Process4Xi-by-repopulate-pool-of-Pool4Xi: " + str(self._initargs) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #()
                    
                    message2print = "{(Step 3/3f) passing" + "; maxtasksperchild-of-pool2xi-passing-to-Process4Xi-by-repopulate-pool-of-Pool4Xi: " + str(self._maxtasksperchild) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #None
                     
                    
                    w2xi = self.Process(target=worker4xi,
                                             args=(self._inqueue,
                                                   self._outqueue,
                                                   self._initializer,
                                                   self._initargs, self._maxtasksperchild)
                                           )
                    message2print = "{(Step 3/3g) got" + "; w2xi-got-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                    #<Process4Xi(Process4Xi-4, initial)>                 
                except:
                    message2print = "{(Step 3/3z3) failed" + "; w2xi-failed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                try:                           
                    self._pool.append(w2xi)
                    message2print = "{(Step 3/3h) appended" + "; append-of-w2xi-appended-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #<Process4Xi(Process4Xi-4, initial)>
                except:
                    message2print = "{(Step 3/3z4) failed" + "; w2xi-failed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                try:
                    message2print = "{(Step 3/3i1) using" + "; name-using-by-repopulate-pool-of-Pool4Xi: " + str(w2xi.name)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # Process4Xi-4
                                                          
                    w2xi.name = w2xi.name.replace('Process', 'PoolWorker')
                    message2print = "{(Step 3/3i2) set" + "; name-set-by-repopulate-pool-of-Pool4Xi to: " + str(w2xi.name)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # PoolWorker4Xi-4 
                except:
                    message2print = "{(Step 3/3z5) failed" + "; name-of-w2xi-failed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #        
                try:    
                    w2xi.daemon = True
                    message2print = "{(Step 3/3j) set" + "; daemon-set-by-repopulate-pool-of-Pool4Xi to: " + str(w2xi.daemon)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #True 
                except:
                    message2print = "{(Step 3/3z6) failed" + "; start-of-w2xi-failed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #                          
                try:    
                    w2xi.start()
                    message2print = "{(Step 3/3k5) performed" + "; start-of-w2xi-performed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi.start)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #<bound method Process4Xi.start of <Process4Xi(PoolWorker4Xi-4, stopped daemon)>> 
                except:
                    message2print = "{(Step 3/3z7) failed" + "; start-of-w2xi-failed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                try:
                    status2workprocess = w2xi.is_alive()    
                    if status2workprocess:
                        message2print = "{(Step 3/3l1) got" + "; alive-status-got-by-repopulate-pool-of-Pool4Xi: " + str(status2workprocess)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # always true
                    else:
                        message2print = "{(Step 3/3l2) got" + "; not-alive-status-got-by-repopulate-pool-of-Pool4Xi: " + str(status2workprocess)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #     
                except:
                    message2print = "{(Step 3/3z8) failed" + "; alive-status-of-w2xi-failed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #  
                                      
                # JATKA - TO BE TESTED WHETHER YOU SHOULD NOT CALL RUN METHOD HERE OR SOMEWHERE ELSE! 
                           
                message2print = "{(Step 3/3m1) performing" + "; run-of-w2xi-performing-by-repopulate-pool-of-Pool4Xi: " + str(w2xi.run)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <bound method Process4Xi.run of <Process4Xi(PoolWorker4Xi-1, stopped daemon)>>
                try:    
                    w2xi.run()                                                                                      
                    message2print = "{(Step 3/3m) performed" + "; run-of-w2xi-performed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi.run)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                except:
                    message2print = "{(Step 3/3z9) failed" + "; run-of-w2xi-failed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #               
        except:
            message2print = "{(Step 3/3z1) failed" + "; w2xi-failed-by-repopulate-pool-of-Pool4Xi: " + str(w2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #       

        # -----------------------------------------------------------------------------------------    
        #(Step 3/3) 
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 3/3) returning" + "; repopulate-pool-of-Pool4Xi-returning: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 01:01:53 01/16/13 Pacific Standard Time
        
        return    
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "maintain-pool-of-Pool4Xi" usage = "
    #   (self)
    #">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _maintain_pool(self):
        '''
        Clean up any exited workers and start replacements for them.
        ''' 
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3) 
        # -----------------------------------------------------------------------------------------     
        class2print = "Pool4Xi"
        method2print = "_maintain_pool"
        
        message2print = "{ (Step 1/3) started" + "; maintain-pool-of-Pool4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        #01:01:53 01/16/13 Pacific Standard Time
    
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3) 
        # -----------------------------------------------------------------------------------------          
        ack2exited = False
        
        if self._join_exited_workers():
        
            self._repopulate_pool()
            
            message2print = "{(Step 2/3a) performed" + "; repopulate-pool-of-pool2xi-performed-by-maintain-pool-of-Pool4Xi: " + str(self._repopulate_pool)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #<bound method Pool4Xi._repopulate_pool of <pool4xi.Pool4Xi object at 0x0000000003C0EA58>> 
        else:
            message2print = "{(Step 2/3b) skipped" + "; ack2exited-skipped-by-maintain-pool-of-Pool4Xi: " + str(ack2exited)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        # -----------------------------------------------------------------------------------------    
        # (Step 3/3) 
        # -----------------------------------------------------------------------------------------          
        message2print = "{(Step 3/3) returning" + "; maintain-pool-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)               
        #01:01:55 01/16/13 Pacific Standard Time
               
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "setup-queues-of-Pool4Xi" usage = "
    #   called from __init__
    #">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _setup_queues(self):
        '''
        setups queues    
        ''' 
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/6) 
        # -----------------------------------------------------------------------------------------     
                   
        class2print = "Pool4Xi"
        method2print = "_setup_queues"
        
        message2print = "{ (Step 1/6) started" + "; setup-queues-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #23:42:47 01/15/13 Pacific Standard Time
        

        # -----------------------------------------------------------------------------------------    
        # (Step 2/6) - INQUEUE  
        # -----------------------------------------------------------------------------------------                 
        
        self._inqueue = SimpleQueue4Xi()

        message2print = "{(Step 2/6) got" + "; inqueue-of-pool2xi-got-by-setup-queues-of-Pool4Xi: " + str(self._inqueue)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003A5D9E8>
                         

        # -----------------------------------------------------------------------------------------    
        # (Step 3/6) - OUTQUEUE 
        # -----------------------------------------------------------------------------------------                 
                   
        self._outqueue = SimpleQueue4Xi()
        
        message2print = "{(Step 3/6) got" + "; outqueue-of-pool2xi-got-by-setup-queues-of-Pool4Xi: " + str(self._outqueue)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003A5DA90>
        
           

        # -----------------------------------------------------------------------------------------    
        #(Step 4/6) - QUICK_PUT 
        # -----------------------------------------------------------------------------------------                 
            
        self._quick_put = self._inqueue._writer.send
        
        message2print = "{(Step 4/6) got" + "; quick-put-of-pool2xi-got-by-setup-queues-of-Pool4Xi: " + str(self._quick_put)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #<bound method PipeConnection4Xi.send of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003A5D898>>
        

        # -----------------------------------------------------------------------------------------    
        #(Step 5/6) - QUICK_GET 
        # -----------------------------------------------------------------------------------------                 
                   
        self._quick_get = self._outqueue._reader.recv
        
        message2print = "{(Step 5/6) got" + "; quick-get-of-pool2xi-got-by-setup-queues-of-Pool4Xi: " + str(self._quick_get)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #<bound method PipeConnection4Xi.recv of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003A5DB38>>
        
        # -----------------------------------------------------------------------------------------    
        #(Step 6/6) 
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 6/6) returning" + "; setup-queues-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        #23:42:47 01/15/13 Pacific Standard Time
    #</methodize4xi>
            

                   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "apply-of-Pool4Xi" usage = "
    #   managers.py
    #       PoolProxy = MakeProxyType('PoolProxy', (
    #             'apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join',">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def apply(self, func, args=(), kwds={}):
        '''
        is equivalent of apply() builtin
        '''
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------     
        class2print = "Pool4Xi"
        method2print = "apply"
        
        message2print = "{(Step 1/3a) started" + "; apply-of-Pool4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        #  
        
        message2print = "{(Step 1/3b) passed" + "; func-passed-to-apply-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        #  
                
        message2print = "{(Step 1/3c) passed" + "; args-passed-to-apply-of-Pool4Xi: " + str(args) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        #  
                     
        message2print = "{(Step 1/3d) passed" + "; kwds-passed-to-apply-of-Pool4Xi: " + str(kwds) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        #  
             
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------                 
        if self._state == Pool4Xi._cp2xi.get_run(): #RUN_POOL4XI dragonized
            message2print = "{(Step 2/3a) accepted" + "; state-of-pool2xi-accepted-by-apply-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
                                                  
        else:
            message2print = "{(Step 2/3b) warned" + "; state-of-pool2xi-warned-by-apply-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
                           

        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------                 
   
        message2print = "{(Step 3/3a ) passing" + "; func-passing-to-apply-async-by-apply-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #  
                             
        message2print = "{(Step 3/3b ) passing" + "; args-passing-to-apply-async-by-apply-of-Pool4Xi: " + str(args) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #  
                                        
        message2print = "{(Step 3/3c ) passing" + "; kwds-passing-to-apply-async-by-apply-of-Pool4Xi: " + str(kwds) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #  
                        
        message2print = "{(Step 3/3d ) returning" + "; apply-async-function-of-pool2xi-returning-by-apply-of-Pool4Xi: " + str(self.apply_async) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        #  
                                     
                
        return self.apply_async(func, args, kwds).get()
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map-of-Pool4Xi" usage = "
    #        managers.py
    #            PoolProxy = MakeProxyType('PoolProxy', ('apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join','map', 'map_async', 'terminate'))">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def map(self, func, iterable, chunksize=None):
        '''
        is equivalent of map() builtin
        '''
    
        # -----------------------------------------------------------------------------------------    
        #(Step 1/3)
        # -----------------------------------------------------------------------------------------     
                
        class2print = "Pool4Xi"
        method2print = "map"
        
        message2print = "{(Step 1/3a) started" + "; map-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        message2print = "{(Step 1/3b) passed" + "; func-passed-to-map-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3c) passed" + "; iterable-passed-to-map-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
           
        message2print = "{(Step 1/3d) passed" + "; chunksize-passed-to-map-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
          
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------                 
        if self._state == Pool4Xi._cp2xi.get_run():                       #RUN_POOL4XI dragonized
            message2print = "{(Step 2/3a) accepted" + "; state-of-pool2xi-accepted-by-map-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #         
        else:
            message2print = "{(Step 2/3b) warned" + "; state-of-pool2xi-warned-by-map-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
                   

        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------                 
            
        message2print = "{(Step 3/3a) passing" + "; func-passing-to-map-async-by-map-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                
        message2print = "{(Step 3/3b) passing" + "; iterable-passing-to-map-async-by-map-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
        message2print = "{(Step 3/3c) passing" + "; chunksize-passing-to-map-async-by-map-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                                
        message2print = "{(Step 3/3d) returning" + "; map-async-function-of-pool2xi-returning-by-map-of-Pool4Xi: " + str(self.map_async) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        #                 

        return self.map_async(func, iterable, chunksize).get()
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "starmap-of-Pool4Xi" usage = "
    #   (self, func, iterable, chunksize=None)
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def starmap(self, func, iterable, chunksize=None):
        '''
        Like `map()` method but the elements of the `iterable` are expected to
        be iterables as well and will be unpacked as arguments. Hence
        `func` and (a, b) becomes func(a, b).
        '''
    
        # -----------------------------------------------------------------------------------------    
        #(Step 1/3)
        # -----------------------------------------------------------------------------------------     
                
        class2print = "Pool4Xi"
        method2print = "starmap"
        
        message2print = "{(Step 1/3a) started" + "; starmap-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        message2print = "{(Step 1/3b) passed" + "; func-passed-to-starmap-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3c) passed" + "; iterable-passed-to-starmap-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #   
   
        message2print = "{(Step 1/3d) passed" + "; chunksize-passed-to-starmap-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------                 
        if self._state == Pool4Xi._cp2xi.get_run():                       #RUN_POOL4XI dragonized
            message2print = "{(Step 2/3a) accepted" + "; state-of-pool2xi-accepted-by-starmap-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #         
        else:
            message2print = "{(Step 2/3b) warned" + "; state-of-pool2xi-warned-by-starmap-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            #        

        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------                 
            
        message2print = "{(Step 3/3a) passing" + "; func-passing-to-starmap-async-by-starmap-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                
        message2print = "{(Step 3/3b) passing" + "; iterable-passing-to-starmap-async-by-starmap-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
        message2print = "{(Step 3/3c) passing" + "; chunksize-passing-to-starmap-async-by-starmap-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        message2print = "{(Step 3/3d) passing" + "; starmapstar-passing-to-starmap-async-by-starmap-of-Pool4Xi: " + str(starmapstar) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
                                
        message2print = "{(Step 3/3e) returning" + "; starmap-async-function-of-pool2xi-returning-by-starmap-of-Pool4Xi: " + str(self.starmap_async) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # 
                        
        return self._map_async(func, iterable, starmapstar, chunksize).get()
        
    #</methodize4xi> 
    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "starmap-async-of-Pool4Xi" usage = "
    #   (self, func, iterable, chunksize=None, callback=None, error_callback=None)
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def starmap_async(self, func, iterable, chunksize=None, callback=None, error_callback=None):
        '''
        Asynchronous version of `starmap()` method
        '''
    
        # -----------------------------------------------------------------------------------------    
        #(Step 1/3)
        # -----------------------------------------------------------------------------------------     
                
        class2print = "Pool4Xi"
        method2print = "starmap_async"
        
        message2print = "{(Step 1/3a) started" + "; starmap-async-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        message2print = "{(Step 1/3b) passed" + "; func-passed-to-starmap-async-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3c) passed" + "; iterable-passed-to-starmap-async-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3d) passed" + "; chunksize-passed-to-starmap-async-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3e) passed" + "; callback-passed-to-starmap-async-of-Pool4Xi: " + str(callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3f) passed" + "; error_callback-passed-to-starmap-async-of-Pool4Xi: " + str(error_callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
          
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------                 
        if self._state == Pool4Xi._cp2xi.get_run():                       #RUN_POOL4XI dragonized
            message2print = "{(Step 2/3a) accepted" + "; state-of-pool2xi-accepted-by-starmap-async-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #         
        else:
            message2print = "{(Step 2/3b) warned" + "; state-of-pool2xi-warned-by-starmap-async-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
                   

        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------                 
            
        message2print = "{(Step 3/3a) passing" + "; func-passing-to-map-async-by-starmap-async-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                
        message2print = "{(Step 3/3b) passing" + "; iterable-passing-to-map-async-by-starmap-async-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
        message2print = "{(Step 3/3c) passing" + "; starmapstar-passing-to-map-async-by-starmap-async-of-Pool4Xi: " + str(starmapstar) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        message2print = "{(Step 3/3d) passing" + "; chunksize-passing-to-map-async-by-starmap-async-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        message2print = "{(Step 3/3e) passing" + "; callback-passing-to-map-async-by-starmap-async-of-Pool4Xi: " + str(callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        message2print = "{(Step 3/3f) passing" + "; error-callback-passing-to-map-async-by-starmap-async-of-Pool4Xi: " + str(error_callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
                                
        message2print = "{(Step 3/3g) returning" + "; map-async-of-pool2xi-returning-by-starmap-of-Pool4Xi: " + str(self._map_async) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # 
        
        return self._map_async(func, iterable, starmapstar, chunksize, callback, error_callback)
    #</methodize4xi>
                   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "imap-of-Pool4Xi" usage = "
    #   managers.py
    #       PoolProxy._method_to_typeid_ = {'apply_async': 'AsyncResult',...'imap': 'Iterator',...">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def imap(self, func, iterable, chunksize=1):
        '''
        Pool4Xi method imap: is equivalent of `itertool.imap()` -- can be MUCH slower than `Pool4Xi.map()`
        '''
    
        # -----------------------------------------------------------------------------------------    
        #(Step 1/9) 
        # -----------------------------------------------------------------------------------------     
                
        class2print = "Pool4Xi"
        method2print = "imap"
        
        message2print = "{(Step 1/9a) started" + "; imap-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        message2print = "{(Step 1/9b) passed" + "; func-passed-to-imap-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
   
        message2print = "{(Step 1/9c) passed" + "; iterable-passed-to-imap-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/9d) passed" + "; chunksize-passed-to-imap-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                
        if self._state == Pool4Xi._cp2xi.get_run():                        #RUN_POOL4XI dragonized
             
            message2print = "{(Step 1/9e) accepted" + "; state-of-pool2xi-accepted-by-imap-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #               
        else:
            message2print = "{(Step 1/9b) warned" + "; state-of-pool2xi-warned-by-imap-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            #     
                            
                        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
           
        if chunksize == 1:
        
            # -----------------------------------------------------------------------------------------    
            # (Step 2/9)
            # -----------------------------------------------------------------------------------------               
            
            message2print = "{(Step 2/9) passing" + "; cache-of-pool2xi-passing-to-IMapIterator4Xi-by-imap-of-Pool4Xi: " + str(self._cache) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     
         
            result2xi = IMapIterator4Xi(self._cache)
        
            # -----------------------------------------------------------------------------------------    
            # (Step 3/9)
            # -----------------------------------------------------------------------------------------               
                
            message2print = "{(Step 3/9a) got" + "; result2xi-got-by-imap-of-Pool4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #     
            message2print = "{(Step 3/9b) passing" + "; result2xi-job-passing-to-taskqueue-put-of-pool2xi-by-imap-of-Pool4Xi: " + str(result2xi._job) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     
                            
            self._taskqueue.put((((result2xi._job, i, func, (x,), {})
                         for i, x in enumerate(iterable)), result2xi._set_length))
                
            # -----------------------------------------------------------------------------------------    
            # (Step 4/9)
            # -----------------------------------------------------------------------------------------                 
               
            message2print = "{(Step 4/9) returning" + "; result2xi-returning-by-imap-of-Pool4Xi: " + str(result2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
            # 
                                                 
            return result2xi
            
        else:
                        
            # -----------------------------------------------------------------------------------------    
            # (Step 5/9)
            # -----------------------------------------------------------------------------------------                 
    
            if chunksize > 1:
            
                message2print = "{(Step 5/9a) asserted" + "; chunksize-asserted-by-imap-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                 
            else:
                message2print = "{(Step 5/9b) warned" + "; chunksize-warned-by-imap-of-Pool4Xi: " + " ** " + str(chunksize)  + " ** " + "; time: " + time4xi() + "}"
                pint4xi(class2print, method2print, message2print)               
                # 
                                
            # -----------------------------------------------------------------------------------------    
            # (Step 6/9)
            # -----------------------------------------------------------------------------------------                 
               
            message2print = "{(Step 6/9a) passing" + "; func-passing-to-Pool4Xi-get-tasks-by-imap-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            message2print = "{(Step 6/9b) passing" + "; iterable-passing-to-Pool4Xi-get-tasks-by-imap-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #         
            
            message2print = "{(Step 6/9c) passing" + "; chunksize-passing-to-Pool4Xi-get-tasks-by-imap-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #                     
            
               
            task_batches2xi = Pool4Xi._get_tasks(func, iterable, chunksize)
                
              
            message2print = "{(Step 6/9d ) set" + "; task_batches2xi-set-by-imap-of-Pool4Xi to: " + str(task_batches2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
                      
            # -----------------------------------------------------------------------------------------    
            # (Step 7/9)
            # -----------------------------------------------------------------------------------------    
    
              
            message2print = "{(Step 7/9b ) passing" + "; cache-of-pool2xi-passing-to-IMapIterator4Xi-by-imap-of-Pool4Xi: " + str(self._cache) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     
                   
            result2xi = IMapIterator4Xi(self._cache)
            
            message2print = "{(Step 7/9c ) got" + "; result2xi-got-by-imap-of-Pool4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                
            # -----------------------------------------------------------------------------------------    
            # (Step 8/9)
            # -----------------------------------------------------------------------------------------                 
                   
            message2print = "{(Step 8/9) passing" + "; result2xi-job-passing-to-taskqueue-put-of-pool2xi-by-imap-of-Pool4Xi: " + str(result2xi._job) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     

            self._taskqueue.put((((result2xi._job, i, mapstar4xi, (x,), {})
                     for i, x in enumerate(task_batches2xi)), result2xi._set_length))
            
                
            # -----------------------------------------------------------------------------------------    
            # (Step 9/9)
            # -----------------------------------------------------------------------------------------                 
               
            message2print = "{(Step 9/9) returning" + "; imap-of-Pool4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #             
       
                     
            return (item for chunk in result2xi for item in chunk)

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "imap-unordered-of-Pool4Xi" usage = "
    #    managers.py
    #        PoolProxy._method_to_typeid_ = {....'imap_unordered': 'Iterator'...">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def imap_unordered(self, func, iterable, chunksize=1):
        '''
        Pool4Xi method imap_unordered: is like `imap()` method but ordering of results is arbitrary
        '''
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/9) 
        # -----------------------------------------------------------------------------------------     
                
        class2print = "Pool4Xi"
        method2print = "imap_unordered"
        
        message2print = "{(Step 1/9a) started" + "; imap-unordered-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
          
        message2print = "{(Step 1/9b) passed" + "; func-passed-to-imap-unordered-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #  
        message2print = "{(Step 1/9c) passed" + "; iterable-passed-to-imap-unordered-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/9d) passed" + "; chunksize-passed-to-imap-unordered-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #       
               
        if self._state == Pool4Xi._cp2xi.get_run():               #RUN_POOL4XI dragonized
        
            message2print = "{(Step 1/9e) asserted" + "; state-of-pool2xi-asserted-by-imap-unordered-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #       
        else:
            message2print = "{(Step 1/9f) warned" + "; state-of-pool2xi-warned-by-imap-unordered-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            #             
            
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
      
        if chunksize == 1:
 
            # -----------------------------------------------------------------------------------------    
            # (Step 2/9) 
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/9a) passing" + "; cache-of-pool2xi-passing-to-IMapUnorderedIterator4Xi-by-imap-unordered-of-Pool4Xi: " + str(self._cache) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     

            result2xi = IMapUnorderedIterator4Xi(self._cache)
            
            message2print = "{(Step 2/9b) got" + "; result2xi-got-by-imap-unordered-of-Pool4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                   
            # -----------------------------------------------------------------------------------------    
            # (Step 3/9)
            # -----------------------------------------------------------------------------------------                 
                       
            message2print = "{(Step 3/9a) passing" + "; result2xi-job-passing-to-taskqueue-put-by-imap-unordered-of-Pool4Xi: " + str(result2xi._job) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     
                              
            message2print = "{(Step 3/9b) passing" + "; func-passing-to-taskqueue-put-of-pool2xi-by-imap-unordered-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #                   
      
            self._taskqueue.put((((result2xi._job, i, func, (x,), {})
                         for i, x in enumerate(iterable)), result2xi._set_length))
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/9)
            # -----------------------------------------------------------------------------------------                 
                
            message2print = "{(Step 4/9a) performed" + "; taskqueue-put-of-pool2xi-performed-by-imap-unordered-of-Pool4Xi: " + str(self._taskqueue.put) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # 
            
            message2print = "{(Step 4/9b) returning" + "; result2xi-returning-by-imap-unordered-of-Pool4Xi: " + str(result2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
            # 
                                                           
            return result2xi
        else:
            # -----------------------------------------------------------------------------------------    
            # (Step 5/9)
            # -----------------------------------------------------------------------------------------                 
            if chunksize > 1:
            
                message2print = "{(Step 5/9) asserted" + "; chunksize-asserted-by-imap-unordered-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
                
            # -----------------------------------------------------------------------------------------    
            # (Step 6/9)
            # -----------------------------------------------------------------------------------------                 
                
            message2print = "{(Step 6/9a) passing" + "; func-passing-to-Pool4Xi-get-tasks-by-imap-unordered-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     
            
            message2print = "{(Step 6/9b) passing" + "; iterable-passing-to-Pool4Xi-get-tasks-by-imap-unordered-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            message2print = "{(Step 6/9c) passing" + "; chunksize-passing-to-Pool4Xi-get-tasks-by-imap-unordered-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #                         
            
            task_batches2xi = Pool4Xi._get_tasks(func, iterable, chunksize)
            
            # -----------------------------------------------------------------------------------------    
            # (Step 7/9)
            # -----------------------------------------------------------------------------------------                 
                
            message2print = "{(Step 7/9a) got" + "; task-batches2xi-got-by-imap-unordered-of-Pool4Xi: " + str(task_batches2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                     
            message2print = "{(Step 7/9b) passing" + "; cache-of-pool2xi-passing-to-IMapUnorderedIterator4Xi-by-imap-unordered-of-Pool4Xi: " + str(self._cache) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     
                            
            result2xi = IMapUnorderedIterator4Xi(self._cache)
            
            # -----------------------------------------------------------------------------------------    
            # (Step 8/9)
            # -----------------------------------------------------------------------------------------                 
    
            message2print = "{(Step 8/9a) got" + "; result2xi-got-by-imap-unordered-of-Pool4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                      
            message2print = "{(Step 8/9b) passing" + "; result2xi-job-passing-to-taskqueue-put-of-pool2xi-by-imap-unordered-of-Pool4Xi: " + str(result2xi._job) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #     
                             
                        
            self._taskqueue.put((((result2xi._job, i, mapstar4xi, (x,), {})
                     for i, x in enumerate(task_batches)), result2xi._set_length))

            
            # -----------------------------------------------------------------------------------------    
            # (Step 9/9)
            # -----------------------------------------------------------------------------------------                 
                
            message2print = "{(Step 9/9a) performed" + "; _taskqueue-put-of-pool2xi-performed-by-imap-unordered-of-Pool4Xi: " + str(self._taskqueue.put) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #            
                                                      
            message2print = "{(Step 9/9b) returning" + "; imap-unordered-of-Pool4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #              

            
            return (item for chunk in result2xi for item in chunk)

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "apply-async-of-Pool4Xi" usage = "#        managers.py
    #            PoolProxy._method_to_typeid_ = {'apply_async': 'AsyncResult',....">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def apply_async(self, func, args=(), kwds={}, callback=None, error_callback=None):
        '''
        Pool4Xi method apply_async: is Asynchronous equivalent of `apply()` builtin
        '''
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/4) 
        # -----------------------------------------------------------------------------------------     
                
        class2print = "Pool4Xi"
        method2print = "apply_async"
        
        message2print = "{(Step 1/4a) started" + "; apply-async-of-Pool4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        #01:02:07 01/16/13 Pacific Standard Time
        
    
        message2print = "{(Step 1/4b) passed" + "; func-passed-to-apply-async-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #<function expxi2 at 0x0000000003B41A60>
    
        message2print = "{(Step 1/4c) passed" + "; args-passed-to-apply-async-of-Pool4Xi: " + str(args) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #(20,)
    
        message2print = "{(Step 1/4d) passed" + "; kwds-passed-to-apply-async-of-Pool4Xi: " + str(kwds) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #{}
    
        message2print = "{(Step 1/4e) passed" + "; callback-passed-to-apply-async-of-Pool4Xi: " + str(callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #None
        
        message2print = "{(Step 1/4f) passed" + "; error-callback-passed-to-apply-async-of-Pool4Xi: " + str(error_callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #None          
               
        
        if self._state == Pool4Xi._cp2xi.get_run():               #RUN_POOL4XI dragonized

            message2print = "{(Step 1/4f) accepted" + "; state-of-pool2xi-accepted-by-apply-async-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #0
               
        else:
            message2print = "{(Step 1/4g) warned" + "; state-of-pool2xi-warned-by-apply-async-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
                           
        # -----------------------------------------------------------------------------------------    
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 2/4a) passing" + "; cache-of-pool2xi-passing-to-ApplyResult4Xi-by-apply-async-of-Pool4Xi: " + str(self._cache) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #{}
         
        message2print = "{(Step 2/4b) passing" + "; callback-passing-to-ApplyResult4Xi-by-apply-async-of-Pool4Xi: " + str(callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        #None
        
        message2print = "{(Step 2/4c) passing" + "; error_callback-passing-to-ApplyResult4Xi-by-apply-async-of-Pool4Xi: " + str(error_callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        #None
                         
        result2xi = ApplyResult4Xi(self._cache, callback, error_callback)
        
        message2print = "{(Step 2/4c) got" + "; result2xi-got-by-apply-async-of-Pool4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #<applyresult4xi.ApplyResult4Xi object at 0x0000000003A16BE0>
                         
        # -----------------------------------------------------------------------------------------    
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------             
        message2print = "{(Step 3/4a) passing" + "; result2xi-job-passing-to-taskqueue-put-of-pool2xi-by-apply-async-of-Pool4Xi: " + str(result2xi._job) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #0
         
        message2print = "{(Step 3/4b) passing" + "; func-passing-to-taskqueue-put-of-pool2xi-by-apply-async-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        #<function expxi2 at 0x0000000003B41A60>
         
        message2print = "{(Step 3/4c) passing" + "; args-passing-to-taskqueue-put-of-pool2xi-by-apply-async-of-Pool4Xi: " + str(args) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #(20,)
         
        message2print = "{(Step 3/4d) passing" + "; kwds-passing-to-taskqueue-put-of-pool2xi-by-apply-async-of-Pool4Xi: " + str(kwds) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                    
        #{}
         
        self._taskqueue.put(([(result2xi._job, None, func, args, kwds)], None))

        message2print = "{(Step 3/4e) performed" + "; taskqueue-put-of-pool2xi-performed-by-apply-async-of-Pool4Xi: " + str(self._taskqueue.put) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        #<bound method Queue.put of <queue.Queue object at 0x0000000003C0EC50>>
                      
        
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 4/4) returning" + "; result2xi-returning-by-apply-async-of-Pool4X: " + str(result2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        #<applyresult4xi.ApplyResult4Xi object at 0x0000000003A16BE0>
        
        return result2xi
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map-async-of-Pool4Xi" usage = "
    #   (self, func, iterable, chunksize=None, callback=None, error_callback=None)
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def map_async(self, func, iterable, chunksize=None, callback=None, error_callback=None):
        '''
        Asynchronous version of `map()` method.
        '''
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------     
        class2print = "Pool4Xi"
        method2print = "map_async"
        
        message2print = "{(Step 2/3a) started" + "; map-async-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        message2print = "{(Step 2/3b) passed" + "; func-passed-to-map-async-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
   
        message2print = "{(Step 2/3c) passed" + "; iterable-passed-to-map-async-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
   
        message2print = "{(Step 2/3d) passed" + "; chunksize-passed-to-map-async-of-Pool4Xi: " + str(chunksize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
   
        message2print = "{(Step 2/3e) passed" + "; callback-passed-to-map-async-of-Pool4Xi: " + str(callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 2/3f) passed" + "; error-callback-passed-to-map-async-of-Pool4Xi: " + str(error_callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                                 
                
        if self._state == Pool4Xi._cp2xi.get_run():               #RUN_POOL4XI dragonized
        
            message2print = "{(Step 2/3f) asserted" + "; state-of-pool2xi-asserted-by-map-async-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
        else:
            message2print = "{(Step 2/3g) warned" + "; state-of-pool2xi-warned-by-map-async-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
         
        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 3/3) returning" + "; map-async-of-pool2xi-returning-by-map-async-of-Pool4Xi: " + str(self._map_async) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        
        return self._map_async(func, iterable, mapstar, chunksize)
        
    #</methodize4xi>
      
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map-async-of-Pool4Xi" usage = "
    #   (self, func, iterable, chunksize=None, callback=None, error_callback=None)
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _map_async(self, func, iterable, mapper, chunksize=None, callback=None, error_callback=None):
        '''
        Helper function to implement map, starmap and their async counterparts.
        '''
    
        # -----------------------------------------------------------------------------------------    
        #(Step 1/8)
        # -----------------------------------------------------------------------------------------     
                
        class2print = "Pool4Xi"
        method2print = "_map_async"
        
        message2print = "{(Step 1/8a) started" + "; map-async-of-Pool4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        message2print = "{(Step 1/8b) passed" + "; func-passed-to-map-async-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
   
        message2print = "{(Step 1/8c) passed" + "; iterable-passed-to-map-async-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 1/8d) passed" + "; iterable-passed-to-map-async-of-Pool4Xi: " + str(iterable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        message2print = "{(Step 1/8e) passed" + "; mapper-passed-to-map-async-of-Pool4Xi: " + str(mapper) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 1/8f) passed" + "; callback-passed-to-map-async-of-Pool4Xi: " + str(callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 1/8g) passed" + "; error_callback-passed-to-map-async-of-Pool4Xi: " + str(error_callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        
          
        # -----------------------------------------------------------------------------------------    
        # (Step 2/8)
        # -----------------------------------------------------------------------------------------                 
        if not hasattr(iterable, '__len__'):
            iterable = list(iterable)         
                   

        # -----------------------------------------------------------------------------------------    
        # (Step 3/8)
        # -----------------------------------------------------------------------------------------                 
        if chunksize is None:
            chunksize, extra2xi = divmod(len(iterable), len(self._pool) * 4)
            message2print = "{(Step 3/8a) got" + "; chunksize-got-by-map-async-of-Pool4Xi: " + str(chunksize)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            message2print = "{(Step 3/8b) got" + "; extra2xi-got-by-map-async-of-Pool4Xi: " + str(extra2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if extra2xi:
                chunksize += 1

        # -----------------------------------------------------------------------------------------    
        # (Step 4/8)
        # -----------------------------------------------------------------------------------------
        if len(iterable) == 0:
            chunksize = 0

        # -----------------------------------------------------------------------------------------    
        # (Step 5/8)
        # -----------------------------------------------------------------------------------------
        task_batches2xi = None
        try:
            task_batches2xi = Pool4Xi._get_tasks(func, iterable, chunksize)
            
            message2print = "{(Step 5/8a) got" + "; task-batches2xi-got-by-map-async-of-Pool4Xi: " + str(task_batches2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
        except:
            message2print = "{(Step 5/8z) failed" + "; task-batches2xi-failed-by-map-async-of-Pool4Xi: " + str(task_batches2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            
        # -----------------------------------------------------------------------------------------    
        # (Step 6/8)
        # -----------------------------------------------------------------------------------------
        result2xi = None
        try:
            result2xi = MapResult4Xi(self._cache, chunksize, len(iterable), callback, error_callback=error_callback)
    
            message2print = "{(Step 6/8a) got" + "; result2xi-got-by-map-async-of-Pool4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                   
        except:
            message2print = "{(Step 6/8z) failed" + "; result2xi-failed-by-map-async-of-Pool4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                                       
        # -----------------------------------------------------------------------------------------    
        # (Step 7/8)
        # -----------------------------------------------------------------------------------------                        
        try:
            self._taskqueue.put((((result2xi._job, i2xi, mapper, (x2xi,), {})
                                  for i2xi, x2xi in enumerate(task_batches2xi)), None))
         
        except:
            message2print = "{(Step 7/8z1) failed" + "; result2xi-failed-by-map-async-of-Pool4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            message2print = "{(Step 7/8z2) failed" + "; task-batches2xi-failed-by-map-async-of-Pool4Xi: " + str(task_batches2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                                                                       
        # -----------------------------------------------------------------------------------------    
        # (Step 8/8)
        # -----------------------------------------------------------------------------------------          
        message2print = "{(Step 3/8g) returning" + "; result2xi-returning-by-starmap-of-Pool4Xi: " + str(result2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        
        return result2xi
    #</methodize4xi>    
    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "handle-workers-of-Pool4Xi" usage = "
    #   (pool)
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    @staticmethod
    def _handle_workers(pool):
        '''
        Keep maintaining workers until the cache gets drained, unless the pool is terminated.
        '''
    
        # -----------------------------------------------------------------------------------------    
        #(Step 1/4)
        # -----------------------------------------------------------------------------------------     
        class2print = "Pool4Xi"
        method2print = "_handle_workers"
        
        message2print = "{(Step 1/4a) started" + "; handle-workers-of-Pool4Xi-started: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)   
        # 01:01:53 01/16/13 Pacific Standard Time; 01:01:57 01/16/13 Pacific Standard Time
        
        message2print = "{(Step 1/4b) passed" + "; pool-passed-to-handle-workers-of-Pool4Xi: " + str(pool) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # <pool4xi.Pool4Xi object at 0x0000000003C0EA58>
        # <pool4xi.Pool4Xi object at 0x0000000003C0ED30>
          
        # -------------------------------------------------------------------------------------------------    
        # (Step 2/4) - Keep maintaining workers until the cache gets drained, unless the pool is terminated
        # -------------------------------------------------------------------------------------------------                 
        thread = threading.current_thread()
        ack2maintain = True
        while thread._state == Pool4Xi._cp2xi.get_run() or (pool._cache and thread._state != Pool4Xi._cp2xi.get_terminate()):
            try:    
                pool._maintain_pool()
                
                message2print = "{(Step 2/4a) performed" + "; maintain-performed-by-handle-workers-of-Pool4Xi: " + str(pool._maintain_pool) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <bound method Pool4Xi._maintain_pool of <pool4xi.Pool4Xi object at 0x0000000003C62A20>> 1/2/.../8
                
                
                time.sleep(0.1)
            except:
                ack2maintain = False
                
                message2print = "{(Step 2/4z) failed" + "; ack2maintain-failed-by-handle-workers-of-Pool4Xi: " + str(ack2maintain) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
           
        # -----------------------------------------------------------------------------------------    
        # (Step 3/4) SEND SENTINEL TO STOP WORKERS JATKA <-- is it ok that it never comes here !?
        # -----------------------------------------------------------------------------------------                 
        try:
            # send sentinel to stop workers
            pool._taskqueue.put(None)
            
            message2print = "{(Step 3/4a) performed" + "; put-performed-by-handle-workers-of-Pool4Xi: " + str(pool._taskqueue.put) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        except:
            message2print = "{(Step 3/4z) failed" + "; put-failed-by-handle-workers-of-Pool4Xi: " + str(None) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                       
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------          
        message2print = "{(Step 4/4) returning" + "; handle-workers-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        
    #</methodize4xi>    
          
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "handle-tasks-of-Pool4Xi" usage = "
    #   pool4xi.py
    #       Pool
    #           __init__
    #">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    @staticmethod
    def _handle_tasks(taskqueue, put, outqueue, pool):
        '''
        handles tasks with queues    
        ''' 
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
        class2print = "Pool4Xi"
        method2print = "_handle_tasks"
        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/5) 
        # -----------------------------------------------------------------------------------------             
        message2print = "{(Step 1/5a) started" + "; handle-tasks-of-Pool4Xi-started: " + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        #00:30:01 01/16/13 Pacific Standard Time
        
        message2print = "{(Step 1/5b) passed" + "; taskqueue-passed-to-handle-tasks-of-Pool4Xi: " + str(taskqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #<queue.Queue object at 0x0000000003DD2C50>
          
        message2print = "{(Step 1/5c) passed" + "; put-passed-to-handle-tasks-of-Pool4Xi: " + str(put) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #<bound method PipeConnection4Xi.send of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003DD2A90>>
          
        message2print = "{(Step 1/5d) passed" + "; outqueue-passed-to-handle-tasks-of-Pool4Xi: " + str(outqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003DD2B70>
          
        message2print = "{(Step 1/5e) passed" + "; pool-passed-to-handle-tasks-of-Pool4Xi: " + str(pool) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #[<Process4Xi(PoolWorker4Xi-4, stopped daemon)>, <Process4Xi(PoolWorker4Xi-5, stopped daemon)>]
                     

        # -----------------------------------------------------------------------------------------    
        # (Step 2/5)
        # -----------------------------------------------------------------------------------------                 
                   
        thread2xi = threading.current_thread()
        
        message2print = "{(Step 2/5) got" + "; thread2xi-got-by-handle-tasks-of-Pool4Xi: " + str(thread2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #<Thread(Thread-2, started daemon 3048)>
                                    
          
        for taskseq2xi, set_length2xi in iter(taskqueue.get, None):
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/5)
            # -----------------------------------------------------------------------------------------              
            i = -1
            
            message2print = "{(Step 3/5a) set" + "; taskseq2xi-set-by-handle-tasks-of-Pool4Xi to: " + str(taskseq2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #[(0, None, <function expxi2 at 0x0000000003B41A60>, (20,), {})]
            #[(1, None, <function expxi4 at 0x0000000003B41AE8>, (40,), {})]               
            #[(2, None, <function expxi8 at 0x0000000003B41B70>, (80,), {})]
            
            message2print = "{(Step 3/5b) set" + "; set-length2xi-set-by-handle-tasks-of-Pool4Xi to: " + str(set_length2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #None;None;None
                              
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                
            for i, task in enumerate(taskseq2xi):

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                
                message2print = "{(Step 3/5c) got" + "; index-got-by-handle-tasks-of-Pool4Xi: " + str(i)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #0;0;0
                
                message2print = "{(Step 3/5d) got" + "; task-got-by-handle-tasks-of-Pool4Xi: " + str(task)  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #(0, None, <function expxi2 at 0x0000000003B41A60>, (20,), {})                              
                #(1, None, <function expxi4 at 0x0000000003B41AE8>, (40,), {})
                #(2, None, <function expxi8 at 0x0000000003B41B70>, (80,), {})
                
                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                if thread2xi._state:
      
                    message2print = "{(Step 3/5e) yielded" + "; thread2xi-state-yielded-by-handle-tasks-of-Pool4Xi: " + " ** " + str(thread2xi._state)  + " ** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                     
                    #
                    
                    break                                                                                           # task handler found thread._state != RUN'
                    

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                    
                try:
      
                    message2print = "{(Step 3/5f) passing" + "; task-passing-to-put-by-handle-tasks-of-Pool4Xi: " + str(task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    #(0, None, <function expxi2 at 0x0000000003B41A60>, (20,), {})
                    #(1, None, <function expxi4 at 0x0000000003B41AE8>, (40,), {})
                    #(2, None, <function expxi8 at 0x0000000003B41B70>, (80,), {})
                    
                    put(task)
                    
                                                                                                                    
                    message2print = "{(Step 3/5g) performed" + "; put-performed-by-handle-tasks-of-Pool4Xi: " + str(put) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    #<bound method PipeConnection4Xi.send of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003A168D0>>
                    #<bound method PipeConnection4Xi.send of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003A16160>>
                    #<bound method PipeConnection4Xi.send of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003C0E9B0>>
                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                                        
                except IOError:

                    message2print = "{(Step 3/5h) hit" + "; IOError-hit-by-handle-tasks-of-Pool4Xi: " + " *** Pool4Xi: _handle_tasks: could not put task on queue *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #  
                                      

                    break
            else:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                if set_length2xi:

                    message2print = "{(Step 3/5i) informing" + "; increase-informing-by-handle-tasks-of-Pool4Xi: " + "Pool4Xi: _handle_tasks: doing set_length() " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                         
                    #  
                    
                    set_length2xi(i+1)
                    
                continue
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                    
            break
        else:
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            message2print = "{(Step 3/5j) got" + "; sentinel-got-by-handle-tasks-of-Pool4Xi: " + "sentinel " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #          
                

        
        # -----------------------------------------------------------------------------------------    
        # (Step 4/5)
        # -----------------------------------------------------------------------------------------                 

        try:
   
            # -------------------------------------------------------------------------------------    
            # TELL RESULT HANDLER TO FINISH WHEN CACHE IS EMPTY
            # -------------------------------------------------------------------------------------     
           
            message2print = "{(Step 4/5a) passing" + "; None-passing-to-outqueue-put-by-handle-tasks-of-Pool4Xi: " + str(None) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #   
            
            outqueue.put(None)
  
    
            message2print = "{(Step 4/5b) performed" + "; outqueue-put-performed-by-handle-tasks-of-Pool4Xi: " + str(outqueue.put) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #       
            
            
            message2print = "{(Step 4/5c) informing" + "; workers-informing-by-handle-tasks-of-Pool4Xi: " + "telling workers there is no more work" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
            # 
            
            # -------------------------------------------------------------------------------------    
            # TELL WORKERS THERE IS NO MORE WORK
            # -------------------------------------------------------------------------------------     
            message2print = "{(Step 4/5d) using" + "; pool-using-by-handle-tasks-of-Pool4Xi: " + str(pool) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #   
            
            for p in pool:
                message2print = "{(Step 4/5e) putting" + "; None-putting-by-handle-tasks-of-Pool4Xi: " + "telling workers there is no more work" + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)              
                #  
                
                put(None)       
                
        except IOError:
            message2print = "{(Step 4/5z) hit" + "; IOError-hit-by-handle-tasks-of-Pool4Xi: " + " *** Pool4Xi task handler got IOError when sending sentinels  *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        # -----------------------------------------------------------------------------------------    
        # (Step 5/5) RETURN - JATKA <-- never comes here
        # -----------------------------------------------------------------------------------------         

        message2print = "{(Step 5/5) returning" + "; handle-tasks-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #              
                
                              
                
                
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "handle-results-of-Pool4Xi" usage = "
    #   ___init___ in in pool.py
    #       self._result_handler = threading.Thread(target=Pool4Xi._handle_results,args=(self._outqueue, self._quick_get, self._cache))">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    @staticmethod
    def _handle_results(outqueue, get, cache):
        '''
        Pool4Xi method _handle_results: handles results with queues and cache     
        ''' 

        # -----------------------------------------------------------------------------------------    
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------           
        class2print = "Pool4Xi"
        method2print = "_handle_results"
        
        message2print = "{(Step 1/6a) started" + "; handle-results-of-Pool4Xi-started: "  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #00:30:01 01/16/13 Pacific Standard Time
        
        message2print = "{(Step 1/6b) passed" + "; outqueue-passed-to-handle-results-of-Pool4Xi: " + str(outqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #<simplequeue4xi.SimpleQueue4Xi object at 0x0000000003DD2B70>
           
        message2print = "{(Step 1/6c) passed" + "; get-passed-to-handle-results-of-Pool4Xi: " + str(get) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #<bound method PipeConnection4Xi.recv of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003DD2C18>>
           
        message2print = "{(Step 1/6d) passed" + "; cache-passed-to-handle-results-of-Pool4Xi: " + str(cache) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #{}
                                    
        # -----------------------------------------------------------------------------------------    
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------                 
        thread2xi = threading.current_thread()

        message2print = "{(Step 2/6) set" + "; thread2xi-set-by-handle-results-of-Pool4Xi to: " + str(thread2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #<Thread(Thread-3, started daemon 2840)>
                        
                              
        # -----------------------------------------------------------------------------------------    
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------                 
           
        
        while 1:
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            
            try:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                task2xi = get()
                
                message2print = "{(Step 3/6a) got" + "; task2xi-got-by-handle-results-of-Pool4Xi: " + str(task2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
                        

                    
            except (IOError, EOFError):

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                message2print = "{(Step 3/6b) hit" + "; error-hit-by-handle-results-of-Pool4Xi: " + " *** handle-results-of-Pool4Xi (Step 3/6b) IOError or EOFError *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
        
                message2print = "{(Step 3/6c) returning" + "; handle-results-of-Pool4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                # 
                                
                                
                return
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
    
            if thread2xi._state:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                if thread2xi._state == Pool4Xi._cp2xi.get_terminate(): #TERMINATE_POOL4XI dragonized
                
                    message2print = "{(Step 3/6d) accepted" + "; thread2xi-state-accepted-by-handle-results-of-Pool4Xi: " + str(thread2xi._state) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    # 
            
                    break
                else:    
                    message2print = "{warned" + "; thread2xi-state-warned-by-handle-results-of-Pool4Xi: " + " ** " + str(thread2xi._state)  + " ** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)               
                    # 
                          
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
    
            if task2xi is None:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                
                message2print = "{(Step 3/6e) got" + "; sentinel-got-by-handle-results-of-Pool4Xi: " + "sentinel " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
        
                break
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
    
            job, i, obj = task2xi
            
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                
            try:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                message2print = "{(Step 3/6f) passing" + "; job-passing-to-cache-by-handle-results-of-Pool4Xi: " + str(job) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 
                        
                message2print = "{(Step 3/6g) passing" + "; i-passing-to-cache-by-handle-results-of-Pool4Xi: " + str(i) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                             
                message2print = "{(Step 3/6h) passing" + "; object-passing-to-cache-by-handle-results-of-Pool4Xi: " + str(obj) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                                                                         
                cache[job]._set(i, obj)

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
        
            except KeyError:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                message2print = "{(Step 3/6i) hit" + "; KeyError-hit-by-handle-results-of-Pool4Xi: " + " *** " + str(KeyError)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                                 
                               
            
                pass

        # -----------------------------------------------------------------------------------------    
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------                 
         

        while cache and thread2xi._state != Pool4Xi._cp2xi.get_terminate():  #TERMINATE_POOL4XI dragonized
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            
        
            try:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                task2xi = get()
                
                message2print = "{(Step 4/6a) got" + "; task2xi-got-by-handle-results-of-Pool4Xi: " + str(task2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
                                             

   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                        
            except (IOError, EOFError):

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                message2print = "{(Step 4/6b) hit" + "; Error-hit-by-handle-results-of-Pool4Xi: " + " *** Pool4Xi:_handle_results got EOFError/IOError *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
             
                message2print = "{(Step 4/6c) returning" + "; handle-results-of-Pool4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                # 
                                                     
                                
                return
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
    
            if task2xi is None:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     

                message2print = "{(Step 4/6d) ignored" + "; task2xi-ignored-by-handle-results-of-Pool4Xi: " + str(task2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
                #
                        
                continue
                
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                    
            job, i, obj = task2xi
            
            message2print = "{(Step 4/6e) got" + "; job-got-by-handle-results-of-Pool4Xi: " + str(job)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                         
            message2print = "{(Step 4/6f) got" + "; i-got-by-handle-results-of-Pool4Xi: " + str(i)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                         
            message2print = "{(Step 4/6g) got" + "; object-got-by-handle-results-of-Pool4Xi: " + str(obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                                                 
            try:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                cache[job]._set(i, obj)
                
                message2print = "{(Step 4/6h) performed" + "; cache-job-performed-by-handle-results-of-Pool4Xi: " + str(cache[job]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                                                            
                                
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                
            except KeyError:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                message2print = "{(Step 4/6i) hit" + "; job-hit-by-handle-results-of-Pool4Xi: " + " *** " + str(job)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                                                 
                                
            
                pass

        # -----------------------------------------------------------------------------------------    
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------                 

        if hasattr(outqueue, '_reader'):
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            
            # If we don't make room available in outqueue then
            # attempts to add the sentinel (None) to outqueue may block.  
            # There is guaranteed to be no more than 2 sentinels.

            message2print = "{(Step 5/6a) checking" + "; outqueue-checking-by-handle-results-of-Pool4Xi: " + str(outqueue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            # 
                                          
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                
            try:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                for i in range(10):
    
                    # -----------------------------------------------------------------------------    
                    #
                    # ----------------------------------------------------------------------------- 
                
                    if not outqueue._reader.poll():
                        break
                    get()
                
                message2print = "{(Step 5/6b) performed" + "; outqueue-checking-performed-by-handle-results-of-Pool4Xi: " + str(outqueue._reader.poll) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                                             
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                         
            except (IOError, EOFError):

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                message2print = "{(Step 5/6c) hit" + "; error-hit-by-handle-results-of-Pool4Xi: " + " *** (Step 5/6c) IOError, EOFError *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                                                 

                pass

        # -----------------------------------------------------------------------------------------    
        # (Step 6/6)
        # ----------------------------------------------------------------------------------------- 
                
        message2print = "{(Step 6/6a) got" + "; len-cache-got-by-handle-results-of-Pool4Xi: " + str(len(cache))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                     
        message2print = "{(Step 6/6b) got" + "; thread2xi-state-got-by-handle-results-of-Pool4Xi: " + str(thread2xi._state)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                                   
        message2print = "{(Step 6/6d) returning" + "; handle-results-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                     
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-tasks-of-Pool4Xi" usage = "
    #   pool.py
    #       imap
    #           task_batches = Pool._get_tasks(func, iterable, chunksize)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    @staticmethod
    def _get_tasks(func, it, size):
        '''
        gets tasks    
        '''

        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------         
        class2print = "Pool4Xi"
        method2print = "_get_tasks"
        
        message2print = "{(Step 1/3a) started" + "; get-tasks-of-Pool4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
      
        message2print = "{(Step 1/3b) passed" + "; func-passed-to-get-tasks-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
      
        message2print = "{(Step 1/3c) passed" + "; it-passed-to-get-tasks-of-Pool4Xi: " + str(it) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
      
        message2print = "{(Step 1/3d) passed" + "; size-passed-to-get-tasks-of-Pool4Xi: " + str(size) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
      
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------                 
            
        it = iter(it)
        
        message2print = "{(Step 2/3a) got" + "; it-got-by-get-tasks-of-Pool4Xi: " + str(it)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                              
        
        while 1:
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            
            x2xi = tuple(itertools.islice(it, size))
        
            message2print = "{(Step 2/3a) got" + "; x2xi-got-by-get-tasks-of-Pool4Xi: " + str(x2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                    
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                
            if not x2xi:
                return
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                
            message2print = "{(Step 2/3b) passing" + "; func-passing-to-by-get-tasks-of-Pool4Xi: " + str(func) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
                      
            
            message2print = "{(Step 2/3c) passing" + "; x2xi-passing-to-by-get-tasks-of-Pool4Xi: " + str(x2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
                                          
                
            yield (func, x2xi)                                  

        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 3/3) returning" + "; get-tasks-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                              
                
                        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reduce-of-Pool4Xi" usage = "reduces Pool">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def __reduce__(self):
        '''
        reduces Pool   
        '''
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
                  
        class2print = "Pool4Xi"
        method2print = "__reduce__"
        
        message2print = "{started" + "; reduce-of-Pool4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
                

        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
                        
        message2print = "{raising" + "; NotImplementedError-raising-by-reduce-of-Pool4Xi: " + "** Pool4Xi:__reduce__: pool objects cannot be passed between processes or pickled ** " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
                       
        
        raise NotImplementedError(
              'Pool4Xi:__reduce__: pool objects cannot be passed between processes or pickled'
              )

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "close-of-Pool4Xi" usage = "closes Pool">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def close(self):
        '''
        Pool4Xi method close: closes Pool   
        '''
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
                  
        class2print = "Pool4Xi"
        method2print = "close"
        
        message2print = "{started" + "; close-of-Pool4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
             

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 

        if self._state == Pool4Xi._cp2xi.get_run():     #RUN_POOL4XI dragonized
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                
            self._state = Pool4Xi._cp2xi.get_close()  #CLOSE_POOL4XI dragonized
            
            message2print = "{set" + "; state-of-pool2xi-set-by-close-of-Pool4Xi to: " + str(self._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            
            self._worker_handler._state = Pool4Xi._cp2xi.get_close()
            
            message2print = "{set" + "; state-of-worker-handler-of-pool2xi-set-by-close-of-Pool4Xi to: " + str(self._worker_handler._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
        else:
            message2print = "{skipped" + "; state-of-worker-handler-of-pool2xi-skipped-by-close-of-Pool4Xi to: " + str(self._worker_handler._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #        
                              
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                                  
        message2print = "{returning" + "; close-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                 
      
    #</methodize4xi>
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "terminate-of-Pool4Xi" usage = "
    #   pool.py
    #       for p in pool:
    #           p.terminate()">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def terminate(self):
        '''
        Pool4Xi method terminate: terminates    
        '''

        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = "Pool4Xi"
        method2print = "terminate"
        
        message2print = "{started" + "; terminate-of-Pool4Xi-started: "  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
               

        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        try:               
            self._state = Pool4Xi._cp2xi.get_terminate()  #TERMINATE_POOL4XI dragonized
            
            message2print = "{set" + "; state-of-pool2xi-set-by-terminate-of-Pool4Xi to: " + str(self._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
                   
            self._worker_handler._state = Pool4Xi._cp2xi.get_terminate()
            
            message2print = "{set" + "; state-of-worker-handler-pool2xi-set-by-terminate-of-Pool4Xi to: " + str(self._worker_handler._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            
            self._terminate()
     
            message2print = "{performed" + "; terminate-of-pool2xi-performed-by-terminate-of-Pool4Xi: " + str(self._terminate) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # 
        except:
            message2print = "{failed" + "; terminate-of-pool2xi-failed-by-terminate-of-Pool4Xi: " + str(None) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #               

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; terminate-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
               
                        
                
                        
         
    #</methodize4xi>
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "join-of-Pool4Xi" usage = "joins to Pool">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def join(self):
        '''
        joins Pool     
        '''

        # -----------------------------------------------------------------------------------------    
        # (Step 1/4)
        # ----------------------------------------------------------------------------------------- 
              
        class2print = "Pool4Xi"
        method2print = "join"
        
        
        message2print = "{(Step 1/4) started" + "; join-of-Pool4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
               
        

        # -----------------------------------------------------------------------------------------    
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------                 
        if (self._state in (Pool4Xi._cp2xi.get_close(),Pool4Xi._cp2xi.get_terminate())): #(CLOSE_POOL4XI, TERMINATE_POOL4XI) dragonized
        
            message2print = "{(Step 2/4a) asserted" + "; state-of-pool2xi-asserted-by-join-of-Pool4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
                                                                            #RUN_POOL4XI
            message2print = "{(Step 2/4b) warned" + "; state-of-pool2xi-warned-by-join-of-Pool4Xi: " + " ** " + str(self._state)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            #          
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        try:
            self._worker_handler.join()
                  
            message2print = "{(Step 2/4c) performed" + "; worker-handler-join-of-pool2xi-performed-by-join-of-Pool4Xi: " + str(self._worker_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                                
        except:
            message2print = "{(Step 2/4d) warned" + "; worker-handler-join-of-pool2xi-warned-by-join-of-pool4xi: " + " ** " + str(self._worker_handler.join)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            #      
                   
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        try:    
        
            self._task_handler.join()
                  
            message2print = "{(Step 2/4e) performed" + "; task-handler-join-of-pool2xi-performed-by-join-of-Pool4Xi: " + str(self._task_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                                
        except:
            message2print = "{(Step 2/4f) warned" + "; task-handler-join-of-pool2xi-warned-by-join-of-pool4xi: " + " ** " + str(self._task_handler.join)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
                               
            
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
        try:    
                                      
            self._result_handler.join()
            
            message2print = "{(Step 2/4g) performed" + "; result-handler-join-of-pool2xi-performed-by-join-of-Pool4Xi: " + str(self._result_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                                
        except:
            message2print = "{(Step 2/4h) warned" + "; resul-handler-join-of-pool2xi-warned-by-join-of-pool4xi: " + " ** " + str(self._resul_handler.join)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                                             
        
        ackcount4xi = 0    
        p2xi = None
        try:    
            # -----------------------------------------------------------------------------------------    
            # (Step 3/4)
            # -----------------------------------------------------------------------------------------         
            for p2xi in self._pool:
       
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------     
               
                message2print = "{(Step 3/4a) joining" + "; pool-joining-by-join-of-Pool4Xi: " + str(p2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                                       
                p2xi.join()
                
                message2print = "{(Step 3/4b) joined" + "; pool-joined-by-join-of-Pool4Xi: " + str(p2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
                       
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------     
                   
                message2print = "{(Step 3/4c) performed" + "; pool-join-performed-by-join-of-Pool4Xi: " + str(p2xi.join) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                       
                ackcount4xi = ackcount4xi +1              
        except:
            message2print = "{(Step 3/4d) warned" + "; pool-of-pool2xi-warned-by-join-of-pool4xi: " + " ** " + str(p2xi)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
                                 
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------                 
        if ackcount4xi > 0:
            message2print = "{(Step 4/4a) measured" + "; ackcount4xi-for-joining-measured-by-join-of-pool4xi: " + str(ackcount4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                       
        else:
            message2print = "{(Step 4/4b) warned" + "; ackcount4xi-for-joining-warned-by-join-of-pool4xi: " + " ** " + str(self._pool)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                          
            # 
               
        message2print = "{(Step 4/4c) returning" + "; join-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                                       
                        
                                
    #</methodize4xi>
            
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "help-stuff-finish-of-Pool4Xi" usage = "helps pool stuff to finish">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    @staticmethod
    def _help_stuff_finish(inqueue, task_handler, size):
        '''
        helps pool stuff to finish     
        ''' 
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
                   
        class2print = "Pool4Xi"
        method2print = "_help_stuff_finish"

        # -----------------------------------------------------------------------------------------    
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 1/4a) started" + "; help-stuff-finish-of-Pool4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
           
        message2print = "{(Step 1/4b) passed" + "; inqueue-passed-to-help-stuff-finish-of-Pool4Xi: " + str(inqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
           
        message2print = "{(Step 1/4c) passed" + "; task-handler-passed-to-help-stuff-finish-of-Pool4Xi: " + str(task_handler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
           
        message2print = "{(Step 1/4d) passed" + "; size-passed-to-help-stuff-finish-of-Pool4Xi: " + str(size) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
             
        # -----------------------------------------------------------------------------------------    
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------                 

        # task_handler may be blocked trying to put items on inqueue
        
        try:
            inqueue._rlock.acquire()
        
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------     
                    
            message2print = "{(Step 2/4a) performed" + "; inqueue-rlock-acquire-performed-by-help-stuff-finish-of-Pool4Xi: " + str(inqueue._rlock.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                          
        except:
            message2print = "{(Step 2/4b) warned" + "; inqueue-rlock-acquire-warned-by-help-stuff-finish-of-Pool4Xi: " + " ** " + str(inqueue._rlock.acquire)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
                                                     

        # -----------------------------------------------------------------------------------------    
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------                 
                
        while task_handler.is_alive() and inqueue._reader.poll():
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            try:    
                inqueue._reader.recv()
                
                message2print = "{(Step 2/4c) received" + "; inqueue-reader-received-by-help-stuff-finish-of-Pool4Xi: " + str(inqueue._reader.recv) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                                    
                
                time.sleep(0)
            except:
                message2print = "{(Step 2/4d) hit" + "; inqueue-reader-recv-hit-by-help-stuff-finish-of-Pool4Xi: " + " *** " + str(inqueue._reader.recv)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                                            
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
    
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 4/4) returning" + "; help-stuff-finish-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                         
                
                            
    #</methodize4xi>
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "terminate-pool-of-Pool4Xi" usage = "terminates pool with tasks, queues, handlers and cache">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    @classmethod
    def _terminate_pool(cls, taskqueue, inqueue, outqueue, pool, worker_handler, task_handler, result_handler, cache):
        '''
        terminates pool with tasks, queues, handlers and cache     
        '''
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/14)
        # -----------------------------------------------------------------------------------------        
        class2print = "Pool4Xi"
        method2print = "_terminate_pool"
        
        message2print = "{(Step 1/14a) started" + "; terminate-pool-of-Pool4Xi-started: "  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #  
        
        message2print = "{(Step 1/14b) passed" + "; class-passed-to-terminate-pool-of-Pool4Xi: " + str(cls) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
     
        message2print = "{(Step 1/14c) passed" + "; taskqueue-passed-to-terminate-pool-of-Pool4Xi: " + str(taskqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
        
        message2print = "{(Step 1/14d) passed" + "; inqueue-passed-to-terminate-pool-of-Pool4Xi: " + str(inqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
        
        message2print = "{(Step 1/14e) passed" + "; outqueue-passed-to-terminate-pool-of-Pool4Xi: " + str(outqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
        
        message2print = "{(Step 1/14f) passed" + "; pool-passed-to-terminate-pool-of-Pool4Xi: " + str(pool) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
        
        message2print = "{(Step 1/14g) passed" + "; worker_handler-passed-to-terminate-pool-of-Pool4Xi: " + str(worker_handler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
        
        message2print = "{(Step 1/14h) passed" + "; task_handler-passed-to-terminate-pool-of-Pool4Xi: " + str(task_handler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
        
        message2print = "{(Step 1/14i) passed" + "; result-handler-passed-to-terminate-pool-of-Pool4Xi: " + str(result_handler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
        
        message2print = "{(Step 1/14j) passed" + "; cache-passed-to-terminate-pool-of-Pool4Xi: " + str(cache) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
                             
        # -----------------------------------------------------------------------------------------    
        # (Step 2/14) - STATES
        # -----------------------------------------------------------------------------------------                 
        worker_handler._state = Pool4Xi._cp2xi.get_terminate()  #TERMINATE_POOL4XI dragonized
        message2print = "{(Step 1/14k) got" + "; worker-handler-state-got-by-terminate-pool-of-Pool4Xi: " + str(worker_handler._state)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        task_handler._state = Pool4Xi._cp2xi.get_terminate
        message2print = "{(Step 1/14l) got" + "; task-handler-state-got-by-terminate-pool-of-Pool4Xi: " + str(task_handler._state)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
                           
                                  
        # -----------------------------------------------------------------------------------------    
        # (Step 3/14) - HELP STUFF TO FINISH
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 3/14a) passing" + "; inqueue-passing-to-cls-help-stuff-finish-by-terminate-pool-of-Pool4Xi: " + str(inqueue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #  
                 
        
        message2print = "{(Step 3/14b) passing" + "; task-handler-passing-to-cls-help-stuff-finish-by-terminate-pool-of-Pool4Xi: " + str(task_handler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #  
                            
        
        message2print = "{(Step 3/14c) passing" + "; len-pool-passing-to-cls-help-stuff-finish-by-terminate-pool-of-Pool4Xi: " + str(len(pool)) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #  
            
        try:
            cls._help_stuff_finish(inqueue, task_handler, len(pool))
        
            message2print = "{(Step 3/14d) performed" + "; cls-help-stuff-finish-performed-by-terminate-pool-of-Pool4Xi: " + str(cls._help_stuff_finish) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
              
            ack4terminate = ack4terminate + 1              
        except:
            message2print = "{(Step 3/14e) warned" + "; cls-help-stuff-finish-warned-by-terminate-pool-of-Pool4Xi: " + " ** " + str(cls._help_stuff_finish)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            #  
                             
            
  
        # -----------------------------------------------------------------------------------------    
        # (Step 4/14)
        # -----------------------------------------------------------------------------------------                 
        if (result_handler.is_alive() or len(cache) == 0):

            message2print = "{(Step 4/14a) asserted" + "; done-asserted-by-terminate-pool-of-Pool4Xi: " + "result handler is alive and cache is not empty" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
             
            ack4terminate = ack4terminate + 1                     
        else:
            if len(cache) == 0:
                message2print = "{(Step 4/14b) warned" + "; length-cache-warned-by-terminate-pool-of-Pool4Xi: " + " ** " + str(len(cache))  + " ** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #  
                                
            else:
                message2print = "{(Step 4/14c) warned" + "; result-handler-dead-warned-by-terminate-pool-of-Pool4Xi: " + " ** " + str(result_handler.is_alive())  + " ** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                             
                #  
            
        # -----------------------------------------------------------------------------------------    
        # (Step 5/14) - RESULT HANDLER STATE AND SENTINEL PUT
        # -----------------------------------------------------------------------------------------                 
        try:
            result_handler._state = Pool4Xi._cp2xi.get_terminate()  #TERMINATE_POOL4XI dragonized
            
            message2print = "{(Step 5/14a) got" + "; result-handler-state-got-by-terminate-pool-of-Pool4Xi: " + str(result_handler._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
                                    
            message2print = "{(Step 5/14b) passing" + "; None-passing-to-outqueue-put-by-terminate-pool-of-Pool4Xi: " + str(None) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #  
                              
    
            outqueue.put(None)                  # sentinel
               
            message2print = "{(Step 5/14c) performed" + "; outqueue-put-performed-by-terminate-pool-of-Pool4Xi: " + str(outqueue.put) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
                   
            ack4terminate = ack4terminate + 1                
        except:
            message2print = "{(Step 5/14c) warned" + "; outqueue-put-warned-by-terminate-pool-of-Pool4Xi: " + " ** " + str(outqueue.put)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            #  
            
        # -----------------------------------------------------------------------------------------              
        # (Step 6/14) WAIT FOR THE WORKER HANDLER TO EXIT BEFORE TERMINATING WORKERS
        # - you don't want workers to be restarted behind your back
        # - join worker handler
        # -----------------------------------------------------------------------------------------   
        if threading.current_thread() is not worker_handler:
            worker_handler.join()
            
            message2print = "{(Step 6/14a) performed" + "; join-performed-by-terminate-pool-of-Pool4Xi: " + str(worker_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
        else:
            message2print = "{(Step 6/14b) skipped" + "; join-skipped-by-terminate-pool-of-Pool4Xi: " + str(worker_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
                

        # -----------------------------------------------------------------------------------------   
        # (Step 7/14) TERMINATE WORKERS WHICH HAVEN'T ALREADY FINISHED
        # ----------------------------------------------------------------------------------------- 
        if pool and hasattr(pool[0], 'terminate'):
            # ------------------------------------------------------------------------------------- 
            # TERMINATING WORKERS
            # ------------------------------------------------------------------------------------- 
            for p2xi in pool:
                message2print = "{(Step 7/14a) using" + "; pool-using-by-terminate-pool-of-Pool4Xi: " + str(p2xi.pid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                #  
                if p2xi.exitcode is None:
                    p2xi.terminate()
                    message2print = "{(Step 7/14b) performed" + "; terminate-performed-by-terminate-pool-of-Pool4Xi: " + str(p2xi.terminate) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    #  
                else:
                    message2print = "{(Step 7/14c) skipped" + "; terminate-skipped-by-terminate-pool-of-Pool4Xi: " + str(p2xi.exitcode) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    #  
                        
        # ----------------------------------------------------------------------------------------- 
        # (Step 8/14) JOINING TASK HANDLER
        # ----------------------------------------------------------------------------------------- 
        if threading.current_thread() is not task_handler:
            task_handler.join()
            message2print = "{(Step 8/14a) performed" + "; join-performed-by-terminate-pool-of-Pool4Xi: " + str(task_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
        else:
            message2print = "{(Step 8/14b) skipped" + "; join-skipped-by-terminate-pool-of-Pool4Xi: " + str(task_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
                

        # -----------------------------------------------------------------------------------------     
        # (Step 9/14) JOINING RESULT HANDLER
        # ----------------------------------------------------------------------------------------- 
        if threading.current_thread() is not result_handler:
            
            result_handler.join()
            message2print = "{(Step 9/14a) performed" + "; join-performed-by-terminate-pool-of-Pool4Xi: " + str(result_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #      
        else:
            message2print = "{(Step 9/14b) skipped" + "; join-skipped-by-terminate-pool-of-Pool4Xi: " + str(result_handler.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
                            
        
        # -----------------------------------------------------------------------------------------    
        # (Step 10/14) - JOINING POOL WORKERS
        # -----------------------------------------------------------------------------------------                 
        if pool and hasattr(pool[0], 'terminate'):
            
            for p2xi in pool:
                message2print = "{(Step 10/14a) using" + "; p2xi-using-by-terminate-pool-of-Pool4Xi: " + str(p2xi.pid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                #  
            
                if p2xi.is_alive():
                    # -----------------------------------------------------------------------------
                    # WORKER HAS NOT YET EXITED
                    # -----------------------------------------------------------------------------
                    message2print = "{(Step 10/14b) using" + "; p2xi-using-by-terminate-pool-of-Pool4Xi: " + str(p2xi.pid) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    #  
                    
                    p2xi.join()
                else:
                    message2print = "{(Step 10/14c) skipped" + "; p2xi-skipped-by-terminate-pool-of-Pool4Xi: " + str(p2xi.pid) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    #  
                        
        else:
            message2print = "{(Step 10/14c) skipped" + "; join-skipped-by-terminate-pool-of-Pool4Xi: " + str(pool) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
                                                          
  
        # -----------------------------------------------------------------------------------------    
        # (Step 14/14)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 14/14b) measured" + "; ack4terminate-measured-by-terminate-pool-of-Pool4Xi: " + str(ack4terminate) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        #  
          

        message2print = "{(Step 14/14b) returning" + "; terminate-pool-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #  
                               
    #</methodize4xi>

                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "enter-pool-of-Pool4Xi" usage = "(self)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def __enter__(self):
        '''
        returns object
        '''
        return self
    #</methodize4xi>        
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "exit-pool-of-Pool4Xi" usage = "(self)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def __exit__(self, exc_type, exc_val, exc_tb):
        '''
        exists
        '''
            
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------        
        class2print = "Pool4Xi"
        method2print = "_terminate_pool"
        
        message2print = "{(Step 1/3a) started" + "; exit-pool-of-Pool4Xi-started: "  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #  

        message2print = "{(Step 1/3b) passed" + "; exc-type-passed-to-exit-pool-of-Pool4Xi: " + str(exc_type) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        # 
        
        message2print = "{(Step 1/3c) passed" + "; exc-val-passed-to-exit-pool-of-Pool4Xi: " + str(exc_val) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        # 
        message2print = "{(Step 1/3d) passed" + "; exc-tb-passed-to-exit-pool-of-Pool4Xi: " + str(exc_tb) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                              
        #  
            
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------     
        ack2terminate = False
        try:
            self.terminate()
        
            ack2terminate = True
            
            message2print = "{(Step 2/3a) performed" + "; terminate-performed-by-terminate-pool-of-Pool4Xi: " + str(self.terminate) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
        except:
            message2print = "{(Step 2/3z) failed" + "; terminate-failed-by-terminate-pool-of-Pool4Xi: " + str(ack2terminate) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #  
            
        # -----------------------------------------------------------------------------------------    
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------    
        message2print = "{(Step 3/3) returning" + "; exit-pool-of-Pool4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #      
        
    #</methodize4xi>
                                           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-Pool4Xi" usage = "senses Pool4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        senses Pool4Xi class objects
        '''
        
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
               
        class2print = 'Pool4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-Pool4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; Pool4Xi-sensing-by-sense-of-Pool4Xi: " + str(Pool4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 
                         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-Pool4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                  
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['taskqueue-of-pool2xi'] = self._taskqueue
            kb2xi['cache-of-pool2xi'] = self._cache             
            kb2xi['pool-of-pool2xi'] = self._pool
            kb2xi['task-handle-of-pool2xi'] = self._task_handle         
            kb2xi['task-handler-daemon-of-pool2xi'] = self._task_handler.daemon
            kb2xi['task-handler-state-of-pool2xi'] = self._task_handler._state             
            kb2xi['result-handler-of-pool2xi'] = self._result_handler
            kb2xi['result-handler-daemon-of-pool2xi'] = self._result_handler.daemon              
            kb2xi['result-handler-state-of-pool2xi'] = self._result_handler._state
            kb2xi['terminate-of-pool2xi'] = self._terminate             
            kb2xi['inqueue-of-pool2xi'] = self._inqueue
            kb2xi['outqueue-of-pool2xi'] = self._outqueue         
            kb2xi['quick-put-of-pool2xi'] = self._quick_put
            kb2xi['quick-get-of-pool2xi'] = self._quick_get             
            kb2xi['state-of-pool2xi'] = self._state
            kb2xi['Process-of-pool2xi'] = self.Process      
            kb2xi['Process-of-pool2xi'] = Pool4Xi.Process
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-Pool4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
              
             
        message2print = "{sensed" + "; Pool4Xi-sensed-by-sense-of-Pool4Xi: " + str(Pool4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-Pool4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        # 
                                   
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-Pool4Xi" usage = "reasons Pool4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons Pool4Xi class objects
        '''
        
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
               
        class2print = 'Pool4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-Pool4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; Pool4Xi-reasoning-by-reason-of-Pool4Xi: " + str(Pool4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
                  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-Pool4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                  
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; Pool4Xi-reasoned-by-reason-of-Pool4Xi: " + str(Pool4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # 
          
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-Pool4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        # 
                                 
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-Pool4Xi" usage = "aces Pool4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces Pool4Xi class objects 
        '''
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
               
        
        class2print = 'Pool4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-Pool4Xi-started: "  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
                
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; Pool4Xi-acing-by-ace-of-Pool4Xi: " + str(Pool4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 
                       
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-Pool4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                  


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; Pool4Xi-aced-by-ace-of-Pool4Xi: " + str(Pool4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                   
        # 
          
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-Pool4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        # 
                                    
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_pool1_mp_xi4.txt
#   sense-pool-one-4xi.xml
#   reason-pool-one-4xi.xml
#   ace-pool-one-4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of pool4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>
